# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Libs
import salt.states.mac_xattr as xattr

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase
from tests.support.mock import (
    MagicMock,
    patch
)


class XAttrTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        return {xattr: {}}

    def test_exists_not(self):
        '''
            Test adding an attribute when it doesn't exist
        '''
        with patch('os.path.exists') as exists_mock:
            expected = {
                'changes': {'key': 'value'},
                'comment': '',
                'name': '/path/to/file',
                'result': True
            }

            exists_mock.return_value = True

            list_mock = MagicMock(return_value={'other.id': 'value2'})
            write_mock = MagicMock()
            with patch.dict(xattr.__salt__, {'xattr.list': list_mock,
                                             'xattr.write': write_mock}):
                out = xattr.exists('/path/to/file', ['key=value'])

                list_mock.assert_called_once_with('/path/to/file')
                write_mock.assert_called_once_with('/path/to/file', 'key', 'value', False)
                self.assertEqual(out, expected)

    def test_exists_change(self):
        '''
            Test changing and attribute value
        '''
        with patch('os.path.exists') as exists_mock:
            expected = {
                'changes': {'key': 'other_value'},
                'comment': '',
                'name': '/path/to/file',
                'result': True
            }

            exists_mock.return_value = True

            list_mock = MagicMock(return_value={'key': 'value'})
            write_mock = MagicMock()
            with patch.dict(xattr.__salt__, {'xattr.list': list_mock,
                                             'xattr.write': write_mock}):
                out = xattr.exists('/path/to/file', ['key=other_value'])

                list_mock.assert_called_once_with('/path/to/file')
                write_mock.assert_called_once_with('/path/to/file', 'key', 'other_value', False)
                self.assertEqual(out, expected)

    def test_exists_already(self):
        '''
            Test that having the same value does nothing
        '''
        with patch('os.path.exists') as exists_mock:
            expected = {
                'changes': {},
                'comment': 'All values existed correctly.',
                'name': '/path/to/file',
                'result': True
            }

            exists_mock.return_value = True

            list_mock = MagicMock(return_value={'key': 'value'})
            write_mock = MagicMock()
            with patch.dict(xattr.__salt__, {'xattr.list': list_mock,
                                             'xattr.write': write_mock}):
                out = xattr.exists('/path/to/file', ['key=value'])

                list_mock.assert_called_once_with('/path/to/file')
                assert not write_mock.called
                self.assertEqual(out, expected)

    def test_delete(self):
        '''
            Test deleting an attribute from a file
        '''
        with patch('os.path.exists') as exists_mock:
            expected = {
                'changes': {'key': 'delete'},
                'comment': '',
                'name': '/path/to/file',
                'result': True
            }

            exists_mock.return_value = True

            list_mock = MagicMock(return_value={'key': 'value2'})
            delete_mock = MagicMock()
            with patch.dict(xattr.__salt__, {'xattr.list': list_mock,
                                             'xattr.delete': delete_mock}):
                out = xattr.delete('/path/to/file', ['key'])

                list_mock.assert_called_once_with('/path/to/file')
                delete_mock.assert_called_once_with('/path/to/file', 'key')
                self.assertEqual(out, expected)

    def test_delete_not(self):
        '''
            Test deleting an attribute that doesn't exist from a file
        '''
        with patch('os.path.exists') as exists_mock:
            expected = {
                'changes': {},
                'comment': 'All attributes were already deleted.',
                'name': '/path/to/file',
                'result': True
            }

            exists_mock.return_value = True

            list_mock = MagicMock(return_value={'other.key': 'value2'})
            delete_mock = MagicMock()
            with patch.dict(xattr.__salt__, {'xattr.list': list_mock,
                                             'xattr.delete': delete_mock}):
                out = xattr.delete('/path/to/file', ['key'])

                list_mock.assert_called_once_with('/path/to/file')
                assert not delete_mock.called
                self.assertEqual(out, expected)



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import os

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, MagicMock, patch

# Import salt libs
import salt.modules.vagrant as vagrant
import salt.exceptions
import salt.utils.platform

TEMP_DATABASE_FILE = '/tmp/salt-tests-tmpdir/test_vagrant.sqlite'


@skipIf(NO_MOCK, NO_MOCK_REASON)
class VagrantTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Unit TestCase for the salt.modules.vagrant module.
    '''
    LOCAL_OPTS = {
                'extension_modules': '',
                'vagrant_sdb_data': {
                    'driver': 'sqlite3',
                    'database': TEMP_DATABASE_FILE,
                    'table': 'sdb',
                    'create_table': True
                    }
                }

    def setup_loader_modules(self):
        vagrant_globals = {
            '__opts__': self.LOCAL_OPTS,
            }
        return {vagrant: vagrant_globals}

    def test_vagrant_get_vm_info_not_found(self):
        mock_sdb = MagicMock(return_value=None)
        with patch.dict(vagrant.__utils__, {'sdb.sdb_get': mock_sdb}):
            with self.assertRaises(salt.exceptions.SaltInvocationError):
                vagrant.get_vm_info('thisNameDoesNotExist')

    def test_vagrant_init_positional(self):
        path_nowhere = os.path.join(os.sep, 'tmp', 'nowhere')
        if salt.utils.platform.is_windows():
            path_nowhere = 'c:{0}'.format(path_nowhere)
        mock_sdb = MagicMock(return_value=None)
        with patch.dict(vagrant.__utils__, {'sdb.sdb_set': mock_sdb}):
            resp = vagrant.init(
                'test1',
                path_nowhere,
                'onetest',
                'nobody',
                False,
                'french',
                {'different': 'very'}
                )
            self.assertTrue(resp.startswith('Name test1 defined'))
            expected = dict(name='test1',
                            cwd=path_nowhere,
                            machine='onetest',
                            runas='nobody',
                            vagrant_provider='french',
                            different='very'
                            )
            mock_sdb.assert_called_with(
                'sdb://vagrant_sdb_data/onetest?{0}'.format(path_nowhere),
                'test1',
                self.LOCAL_OPTS)
            mock_sdb.assert_any_call(
                'sdb://vagrant_sdb_data/test1',
                expected,
                self.LOCAL_OPTS)

    def test_vagrant_get_vm_info(self):
        testdict = {'testone': 'one', 'machine': 'two'}
        mock_sdb = MagicMock(return_value=testdict)
        with patch.dict(vagrant.__utils__, {'sdb.sdb_get': mock_sdb}):
            resp = vagrant.get_vm_info('test1')
            self.assertEqual(resp, testdict)

    def test_vagrant_init_dict(self):
        testdict = dict(cwd='/tmp/anywhere',
                        machine='twotest',
                        runas='somebody',
                        vagrant_provider='english')
        expected = testdict.copy()
        expected['name'] = 'test2'
        mock_sdb = MagicMock(return_value=None)
        with patch.dict(vagrant.__utils__, {'sdb.sdb_set': mock_sdb}):
            vagrant.init('test2', vm=testdict)
            mock_sdb.assert_any_call(
                'sdb://vagrant_sdb_data/test2',
                expected,
                self.LOCAL_OPTS)

    def test_vagrant_init_arg_override(self):
        testdict = dict(cwd='/tmp/there',
                        machine='treetest',
                        runas='anybody',
                        vagrant_provider='spansh')
        mock_sdb = MagicMock(return_value=None)
        with patch.dict(vagrant.__utils__, {'sdb.sdb_set': mock_sdb}):
            vagrant.init('test3',
                        cwd='/tmp',
                        machine='threetest',
                        runas='him',
                        vagrant_provider='polish',
                        vm=testdict)
            expected = dict(name='test3',
                        cwd='/tmp',
                        machine='threetest',
                        runas='him',
                        vagrant_provider='polish')
            mock_sdb.assert_any_call(
                'sdb://vagrant_sdb_data/test3',
                expected,
                self.LOCAL_OPTS)

    def test_vagrant_get_ssh_config_fails(self):
        mock_sdb = MagicMock(return_value=None)
        with patch.dict(vagrant.__utils__, {'sdb.sdb_set': mock_sdb}):
            mock_sdb = MagicMock(return_value={})
            with patch.dict(vagrant.__utils__, {'sdb.sdb_get': mock_sdb}):
                vagrant.init('test3', cwd='/tmp')
                with self.assertRaises(salt.exceptions.SaltInvocationError):
                    vagrant.get_ssh_config('test3')  # has not been started

    def test_vagrant_destroy(self):
        path_mydir = os.path.join(os.sep, 'my', 'dir')
        if salt.utils.platform.is_windows():
            path_mydir = 'c:{0}'.format(path_mydir)
        mock_cmd = MagicMock(return_value={'retcode': 0})
        with patch.dict(vagrant.__salt__, {'cmd.run_all': mock_cmd}):
            mock_sdb = MagicMock(return_value=None)
            with patch.dict(vagrant.__utils__, {'sdb.sdb_delete': mock_sdb}):
                mock_sdb_get = MagicMock(return_value={
                    'machine': 'macfour', 'cwd': path_mydir})
                with patch.dict(vagrant.__utils__, {'sdb.sdb_get': mock_sdb_get}):
                    self.assertTrue(vagrant.destroy('test4'))
                    mock_sdb.assert_any_call(
                        'sdb://vagrant_sdb_data/macfour?{0}'.format(path_mydir),
                        self.LOCAL_OPTS)
                    mock_sdb.assert_any_call(
                        'sdb://vagrant_sdb_data/test4',
                        self.LOCAL_OPTS)
                    cmd = 'vagrant destroy -f macfour'
                    mock_cmd.assert_called_with(cmd,
                                                runas=None,
                                                cwd=path_mydir,
                                                output_loglevel='info')

    def test_vagrant_start(self):
        mock_cmd = MagicMock(return_value={'retcode': 0})
        with patch.dict(vagrant.__salt__, {'cmd.run_all': mock_cmd}):
            mock_sdb_get = MagicMock(return_value={
                'machine': 'five', 'cwd': '/the/dir', 'runas': 'me',
                'vagrant_provider': 'him'})
            with patch.dict(vagrant.__utils__, {'sdb.sdb_get': mock_sdb_get}):
                self.assertTrue(vagrant.start('test5'))
                cmd = 'vagrant up five --provider=him'
                mock_cmd.assert_called_with(cmd,
                                            runas='me',
                                            cwd='/the/dir',
                                            output_loglevel='info')



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Megan Wilhite (mwilhite@saltstack.com)`


    tests.unit.returners.test_syslog_return
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, MagicMock, patch

# Import salt libs
import salt.returners.syslog_return as syslog


@skipIf(NO_MOCK, NO_MOCK_REASON)
class SyslogReturnerTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test Syslog returner
    '''
    def setup_loader_modules(self):
        return {syslog: {}}

    @skipIf(not syslog.HAS_SYSLOG, 'Skip when syslog not installed')
    def test_syslog_returner_unicode(self):
        '''
        test syslog returner with unicode
        '''
        ret = {'fun_args': [], 'jid': '20180713160901624786', 'return': True,
               'retcode': 0, 'success': True, 'fun': 'test.ping', 'id': '02e10e971a30'}
        opts = {u'level': u'LOG_INFO', u'options': [],
                u'facility': u'LOG_USER', u'tag': u'salt-minion'}

        with patch('salt.returners.syslog_return._get_options',
                   MagicMock(return_value=opts)):
            try:
                syslog.returner(ret)
            except Exception as e:
                self.fail('syslog.returner() failed with exception: {0}'.format(e))



# -*- coding: utf-8 -*-
'''
Test the win_wusa execution module
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, patch, MagicMock
from tests.support.unit import TestCase, skipIf

# Import Salt Libs
import salt.utils.platform
import salt.modules.win_wusa as win_wusa
from salt.exceptions import CommandExecutionError


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not salt.utils.platform.is_windows(), 'System is not Windows')
class WinWusaTestCase(TestCase, LoaderModuleMockMixin):
    '''
    test the functions in the win_wusa execution module
    '''
    def setup_loader_modules(self):
        return {win_wusa: {}}

    def test_is_installed_false(self):
        '''
        test is_installed function when the KB is not installed
        '''
        mock_retcode = MagicMock(return_value=1)
        with patch.dict(win_wusa.__salt__, {'cmd.retcode': mock_retcode}):
            self.assertFalse(win_wusa.is_installed('KB123456'))

    def test_is_installed_true(self):
        '''
        test is_installed function when the KB is installed
        '''
        mock_retcode = MagicMock(return_value=0)
        with patch.dict(win_wusa.__salt__, {'cmd.retcode': mock_retcode}):
            self.assertTrue(win_wusa.is_installed('KB123456'))

    def test_list(self):
        '''
        test list function
        '''
        ret = {'pid': 1,
               'retcode': 0,
               'stderr': '',
               'stdout': '[{"HotFixID": "KB123456"}, '
                         '{"HotFixID": "KB123457"}]'}
        mock_all = MagicMock(return_value=ret)
        with patch.dict(win_wusa.__salt__, {'cmd.run_all': mock_all}):
            expected = ['KB123456', 'KB123457']
            returned = win_wusa.list()
            self.assertListEqual(expected, returned)

    def test_install(self):
        '''
        test install function
        '''
        mock_retcode = MagicMock(return_value=0)
        path = 'C:\\KB123456.msu'
        with patch.dict(win_wusa.__salt__, {'cmd.retcode': mock_retcode}):
            self.assertTrue(win_wusa.install(path))
        mock_retcode.assert_called_once_with(
            ['wusa.exe', path, '/quiet', '/norestart'], ignore_retcode=True)

    def test_install_restart(self):
        '''
        test install function with restart=True
        '''
        mock_retcode = MagicMock(return_value=0)
        path = 'C:\\KB123456.msu'
        with patch.dict(win_wusa.__salt__, {'cmd.retcode': mock_retcode}):
            self.assertTrue(win_wusa.install(path, restart=True))
        mock_retcode.assert_called_once_with(
            ['wusa.exe', path, '/quiet', '/forcerestart'], ignore_retcode=True)

    def test_install_already_installed(self):
        '''
        test install function when KB already installed
        '''
        mock_retcode = MagicMock(return_value=2359302)
        path = 'C:\\KB123456.msu'
        name = 'KB123456.msu'
        with patch.dict(win_wusa.__salt__, {'cmd.retcode': mock_retcode}):
            with self.assertRaises(CommandExecutionError) as excinfo:
                win_wusa.install(path)
        mock_retcode.assert_called_once_with(
            ['wusa.exe', path, '/quiet', '/norestart'], ignore_retcode=True)
        self.assertEqual('{0} is already installed'.format(name),
                         excinfo.exception.strerror)

    def test_install_error_87(self):
        '''
        test install function when error 87 returned
        '''
        mock_retcode = MagicMock(return_value=87)
        path = 'C:\\KB123456.msu'
        with patch.dict(win_wusa.__salt__, {'cmd.retcode': mock_retcode}):
            with self.assertRaises(CommandExecutionError) as excinfo:
                win_wusa.install(path)
        mock_retcode.assert_called_once_with(
            ['wusa.exe', path, '/quiet', '/norestart'], ignore_retcode=True)
        self.assertEqual('Unknown error', excinfo.exception.strerror)

    def test_install_error_other(self):
        '''
        test install function on other unknown error
        '''
        mock_retcode = MagicMock(return_value=1234)
        path = 'C:\\KB123456.msu'
        with patch.dict(win_wusa.__salt__, {'cmd.retcode': mock_retcode}):
            with self.assertRaises(CommandExecutionError) as excinfo:
                win_wusa.install(path)
        mock_retcode.assert_called_once_with(
            ['wusa.exe', path, '/quiet', '/norestart'], ignore_retcode=True)
        self.assertEqual('Unknown error: 1234', excinfo.exception.strerror)

    def test_uninstall_kb(self):
        '''
        test uninstall function passing kb name
        '''
        mock_retcode = MagicMock(return_value=0)
        kb = 'KB123456'
        with patch.dict(win_wusa.__salt__, {'cmd.retcode': mock_retcode}), \
                patch("os.path.exists", MagicMock(return_value=False)):
            self.assertTrue(win_wusa.uninstall(kb))
        mock_retcode.assert_called_once_with(
            ['wusa.exe', '/uninstall', '/quiet', '/kb:{0}'.format(kb[2:]), '/norestart'],
            ignore_retcode=True)

    def test_uninstall_path(self):
        '''
        test uninstall function passing full path to .msu file
        '''
        mock_retcode = MagicMock(return_value=0)
        path = 'C:\\KB123456.msu'
        with patch.dict(win_wusa.__salt__, {'cmd.retcode': mock_retcode}), \
                patch("os.path.exists", MagicMock(return_value=True)):
            self.assertTrue(win_wusa.uninstall(path))
        mock_retcode.assert_called_once_with(
            ['wusa.exe', '/uninstall', '/quiet', path, '/norestart'],
            ignore_retcode=True)

    def test_uninstall_path_restart(self):
        '''
        test uninstall function with full path and restart=True
        '''
        mock_retcode = MagicMock(return_value=0)
        path = 'C:\\KB123456.msu'
        with patch.dict(win_wusa.__salt__, {'cmd.retcode': mock_retcode}), \
             patch("os.path.exists", MagicMock(return_value=True)):
            self.assertTrue(win_wusa.uninstall(path, restart=True))
        mock_retcode.assert_called_once_with(
            ['wusa.exe', '/uninstall', '/quiet', path, '/forcerestart'],
            ignore_retcode=True)

    def test_uninstall_already_uninstalled(self):
        '''
        test uninstall function when KB already uninstalled
        '''
        mock_retcode = MagicMock(return_value=2359303)
        kb = 'KB123456'
        with patch.dict(win_wusa.__salt__, {'cmd.retcode': mock_retcode}):
            with self.assertRaises(CommandExecutionError) as excinfo:
                win_wusa.uninstall(kb)
        mock_retcode.assert_called_once_with(
            ['wusa.exe', '/uninstall', '/quiet', '/kb:{0}'.format(kb[2:]), '/norestart'],
            ignore_retcode=True)
        self.assertEqual('{0} not installed'.format(kb),
                         excinfo.exception.strerror)

    def test_uninstall_path_error_other(self):
        '''
        test uninstall function with unknown error
        '''
        mock_retcode = MagicMock(return_value=1234)
        path = 'C:\\KB123456.msu'
        with patch.dict(win_wusa.__salt__, {'cmd.retcode': mock_retcode}), \
                patch("os.path.exists", MagicMock(return_value=True)), \
                self.assertRaises(CommandExecutionError) as excinfo:
            win_wusa.uninstall(path)
        mock_retcode.assert_called_once_with(
            ['wusa.exe', '/uninstall', '/quiet', path, '/norestart'],
            ignore_retcode=True)
        self.assertEqual('Unknown error: 1234', excinfo.exception.strerror)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)


    tests.unit.modules.alternatives_test
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'''

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.helpers import TestsLoggingHandler
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, MagicMock, patch

# Import salt libs
import salt.modules.alternatives as alternatives


@skipIf(NO_MOCK, NO_MOCK_REASON)
class AlternativesTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        return {alternatives: {}}

    def test_display(self):
        with patch.dict(alternatives.__grains__, {'os_family': 'RedHat'}):
            mock = MagicMock(return_value={'retcode': 0, 'stdout': 'salt'})
            with patch.dict(alternatives.__salt__, {'cmd.run_all': mock}):
                solution = alternatives.display('better-world')
                self.assertEqual('salt', solution)
                mock.assert_called_once_with(
                    ['alternatives', '--display', 'better-world'],
                    python_shell=False
                )

        with patch.dict(alternatives.__grains__, {'os_family': 'Suse'}):
            mock = MagicMock(
                return_value={'retcode': 0, 'stdout': 'undoubtedly-salt'}
            )
            with patch.dict(alternatives.__salt__, {'cmd.run_all': mock}):
                solution = alternatives.display('better-world')
                self.assertEqual('undoubtedly-salt', solution)
                mock.assert_called_once_with(
                    ['update-alternatives', '--display', 'better-world'],
                    python_shell=False
                )

        with patch.dict(alternatives.__grains__, {'os_family': 'RedHat'}):
            mock = MagicMock(
                return_value={
                    'retcode': 1,
                    'stdout': 'salt-out',
                    'stderr': 'salt-err'
                }
            )
            with patch.dict(alternatives.__salt__, {'cmd.run_all': mock}):
                solution = alternatives.display('better-world')
                self.assertEqual('salt-err', solution)
                mock.assert_called_once_with(
                    ['alternatives', '--display', 'better-world'],
                    python_shell=False
                )

    def test_show_current(self):
        mock = MagicMock(return_value='/etc/alternatives/salt')
        with patch('salt.utils.path.readlink', mock):
            ret = alternatives.show_current('better-world')
            self.assertEqual('/etc/alternatives/salt', ret)
            mock.assert_called_once_with('/etc/alternatives/better-world')

            with TestsLoggingHandler() as handler:
                mock.side_effect = OSError('Hell was not found!!!')
                self.assertFalse(alternatives.show_current('hell'))
                mock.assert_called_with('/etc/alternatives/hell')
                self.assertIn('ERROR:alternative: hell does not exist',
                              handler.messages)

    def test_check_installed(self):
        mock = MagicMock(return_value='/etc/alternatives/salt')
        with patch('salt.utils.path.readlink', mock):
            self.assertTrue(
                alternatives.check_installed(
                    'better-world', '/etc/alternatives/salt'
                )
            )
            mock.return_value = False
            self.assertFalse(
                alternatives.check_installed(
                    'help', '/etc/alternatives/salt'
                )
            )

    def test_install(self):
        with patch.dict(alternatives.__grains__, {'os_family': 'RedHat'}):
            mock = MagicMock(return_value={'retcode': 0, 'stdout': 'salt'})
            with patch.dict(alternatives.__salt__, {'cmd.run_all': mock}):
                solution = alternatives.install(
                    'better-world',
                    '/usr/bin/better-world',
                    '/usr/bin/salt',
                    100
                )
                self.assertEqual('salt', solution)
                mock.assert_called_once_with(
                    ['alternatives', '--install', '/usr/bin/better-world',
                     'better-world', '/usr/bin/salt', '100'],
                    python_shell=False
                )

        with patch.dict(alternatives.__grains__, {'os_family': 'Debian'}):
            mock = MagicMock(return_value={'retcode': 0, 'stdout': 'salt'})
            with patch.dict(alternatives.__salt__, {'cmd.run_all': mock}):
                solution = alternatives.install(
                    'better-world',
                    '/usr/bin/better-world',
                    '/usr/bin/salt',
                    100
                )
                self.assertEqual('salt', solution)
                mock.assert_called_once_with(
                    ['update-alternatives', '--install', '/usr/bin/better-world',
                     'better-world', '/usr/bin/salt', '100'],
                    python_shell=False
                )

        with patch.dict(alternatives.__grains__, {'os_family': 'RedHat'}):
            mock = MagicMock(
                return_value={
                    'retcode': 1,
                    'stdout': 'salt-out',
                    'stderr': 'salt-err'
                }
            )
            with patch.dict(alternatives.__salt__, {'cmd.run_all': mock}):
                ret = alternatives.install(
                    'better-world',
                    '/usr/bin/better-world',
                    '/usr/bin/salt',
                    100
                )
                self.assertEqual('salt-err', ret)
                mock.assert_called_once_with(
                    ['alternatives', '--install', '/usr/bin/better-world',
                     'better-world', '/usr/bin/salt', '100'],
                    python_shell=False
                )

    def test_remove(self):
        with patch.dict(alternatives.__grains__, {'os_family': 'RedHat'}):
            mock = MagicMock(return_value={'retcode': 0, 'stdout': 'salt'})
            with patch.dict(alternatives.__salt__, {'cmd.run_all': mock}):
                solution = alternatives.remove(
                    'better-world',
                    '/usr/bin/better-world',
                )
                self.assertEqual('salt', solution)
                mock.assert_called_once_with(
                    ['alternatives', '--remove', 'better-world',
                     '/usr/bin/better-world'], python_shell=False
                )

        with patch.dict(alternatives.__grains__, {'os_family': 'Debian'}):
            mock = MagicMock(return_value={'retcode': 0, 'stdout': 'salt'})
            with patch.dict(alternatives.__salt__, {'cmd.run_all': mock}):
                solution = alternatives.remove(
                    'better-world',
                    '/usr/bin/better-world',
                )
                self.assertEqual('salt', solution)
                mock.assert_called_once_with(
                    ['update-alternatives', '--remove', 'better-world',
                     '/usr/bin/better-world'], python_shell=False
                )

        with patch.dict(alternatives.__grains__, {'os_family': 'RedHat'}):
            mock = MagicMock(
                return_value={
                    'retcode': 1,
                    'stdout': 'salt-out',
                    'stderr': 'salt-err'
                }
            )
            with patch.dict(alternatives.__salt__, {'cmd.run_all': mock}):
                solution = alternatives.remove(
                    'better-world',
                    '/usr/bin/better-world',
                )
                self.assertEqual('salt-err', solution)
                mock.assert_called_once_with(
                    ['alternatives', '--remove', 'better-world',
                     '/usr/bin/better-world'], python_shell=False
                )



# -*- coding: utf-8 -*-
'''
    :synopsis: Base class for kernelpkg modules
    :platform: Linux
    :maturity: develop
    .. versionadded:: 2018.3.0
'''
# pylint: disable=invalid-name,no-member

from __future__ import absolute_import, print_function, unicode_literals

# Salt testing libs
try:
    from tests.support.mock import MagicMock, patch
    from salt.exceptions import CommandExecutionError
except ImportError:
    pass


class KernelPkgTestCase(object):
    '''
    Test cases shared by all kernelpkg virtual modules
    '''

    def test_active(self):
        '''
        Test - Return return the active kernel version
        '''
        self.assertEqual(self._kernelpkg.active(), self.KERNEL_LIST[0])

    def test_latest_available_no_results(self):
        '''
        Test - Return the latest available kernel version
        '''
        mock = MagicMock(return_value='')
        with patch.dict(self._kernelpkg.__salt__, {'pkg.latest_version': mock}):
            with patch.object(self._kernelpkg, 'active', return_value=self.KERNEL_LIST[0]):
                self.assertEqual(self._kernelpkg.latest_available(), self.KERNEL_LIST[-1])

    def test_latest_available_at_latest(self):
        '''
        Test - Return the latest available kernel version
        '''
        mock = MagicMock(return_value=self.LATEST)
        with patch.dict(self._kernelpkg.__salt__, {'pkg.latest_version': mock}):
            with patch.object(self._kernelpkg, 'active', return_value=self.KERNEL_LIST[-1]):
                self.assertEqual(self._kernelpkg.latest_available(), self.KERNEL_LIST[-1])

    def test_latest_available_with_updates(self):
        '''
        Test - Return the latest available kernel version
        '''
        mock = MagicMock(return_value=self.LATEST)
        with patch.dict(self._kernelpkg.__salt__, {'pkg.latest_version': mock}):
            with patch.object(self._kernelpkg, 'active', return_value=self.KERNEL_LIST[0]):
                self.assertEqual(self._kernelpkg.latest_available(), self.KERNEL_LIST[-1])

    def test_latest_installed_with_updates(self):
        '''
        Test - Return the latest installed kernel version
        '''
        with patch.object(self._kernelpkg, 'active', return_value=self.KERNEL_LIST[0]):
            with patch.object(self._kernelpkg, 'list_installed', return_value=self.KERNEL_LIST):
                self.assertEqual(self._kernelpkg.latest_installed(), self.KERNEL_LIST[-1])

    def test_latest_installed_at_latest(self):
        '''
        Test - Return the latest installed kernel version
        '''
        with patch.object(self._kernelpkg, 'active', return_value=self.KERNEL_LIST[-1]):
            with patch.object(self._kernelpkg, 'list_installed', return_value=self.KERNEL_LIST):
                self.assertEqual(self._kernelpkg.latest_installed(), self.KERNEL_LIST[-1])

    def test_needs_reboot_with_update(self):
        '''
        Test - Return True if a new kernel is ready to be booted
        '''
        with patch.object(self._kernelpkg, 'active', return_value=self.KERNEL_LIST[0]):
            with patch.object(self._kernelpkg, 'latest_installed', return_value=self.KERNEL_LIST[1]):
                self.assertTrue(self._kernelpkg.needs_reboot())

    def test_needs_reboot_at_latest(self):
        '''
        Test - Return True if a new kernel is ready to be booted
        '''
        with patch.object(self._kernelpkg, 'active', return_value=self.KERNEL_LIST[1]):
            with patch.object(self._kernelpkg, 'latest_installed', return_value=self.KERNEL_LIST[1]):
                self.assertFalse(self._kernelpkg.needs_reboot())

    def test_needs_reboot_order_inverted(self):
        '''
        Test - Return True if a new kernel is ready to be booted
        '''
        with patch.object(self._kernelpkg, 'active', return_value=self.KERNEL_LIST[1]):
            with patch.object(self._kernelpkg, 'latest_installed', return_value=self.KERNEL_LIST[0]):
                self.assertFalse(self._kernelpkg.needs_reboot())

    def test_upgrade_not_needed_with_reboot(self):
        '''
        Test - Upgrade function when no upgrade is available and reboot has been requested
        '''
        with patch.object(self._kernelpkg, 'active', return_value=self.KERNEL_LIST[-1]):
            with patch.object(self._kernelpkg, 'list_installed', return_value=self.KERNEL_LIST):
                result = self._kernelpkg.upgrade(reboot=True)
                self.assertIn('upgrades', result)
                self.assertEqual(result['active'], self.KERNEL_LIST[-1])
                self.assertEqual(result['latest_installed'], self.KERNEL_LIST[-1])
                self.assertEqual(result['reboot_requested'], True)
                self.assertEqual(result['reboot_required'], False)
                self._kernelpkg.__salt__['system.reboot'].assert_not_called()

    def test_upgrade_not_needed_without_reboot(self):
        '''
        Test - Upgrade function when no upgrade is available and no reboot has been requested
        '''
        with patch.object(self._kernelpkg, 'active', return_value=self.KERNEL_LIST[-1]):
            with patch.object(self._kernelpkg, 'list_installed', return_value=self.KERNEL_LIST):
                result = self._kernelpkg.upgrade(reboot=False)
                self.assertIn('upgrades', result)
                self.assertEqual(result['active'], self.KERNEL_LIST[-1])
                self.assertEqual(result['latest_installed'], self.KERNEL_LIST[-1])
                self.assertEqual(result['reboot_requested'], False)
                self.assertEqual(result['reboot_required'], False)
                self._kernelpkg.__salt__['system.reboot'].assert_not_called()

    def test_upgrade_needed_with_reboot(self):
        '''
        Test - Upgrade function when an upgrade is available and reboot has been requested
        '''
        with patch.object(self._kernelpkg, 'active', return_value=self.KERNEL_LIST[0]):
            with patch.object(self._kernelpkg, 'list_installed', return_value=self.KERNEL_LIST):
                result = self._kernelpkg.upgrade(reboot=True)
                self.assertIn('upgrades', result)
                self.assertEqual(result['active'], self.KERNEL_LIST[0])
                self.assertEqual(result['latest_installed'], self.KERNEL_LIST[-1])
                self.assertEqual(result['reboot_requested'], True)
                self.assertEqual(result['reboot_required'], True)
                self.assert_called_once(self._kernelpkg.__salt__['system.reboot'])

    def test_upgrade_needed_without_reboot(self):
        '''
        Test - Upgrade function when an upgrade is available and no reboot has been requested
        '''
        with patch.object(self._kernelpkg, 'active', return_value=self.KERNEL_LIST[0]):
            with patch.object(self._kernelpkg, 'list_installed', return_value=self.KERNEL_LIST):
                result = self._kernelpkg.upgrade(reboot=False)
                self.assertIn('upgrades', result)
                self.assertEqual(result['active'], self.KERNEL_LIST[0])
                self.assertEqual(result['latest_installed'], self.KERNEL_LIST[-1])
                self.assertEqual(result['reboot_requested'], False)
                self.assertEqual(result['reboot_required'], True)
                self._kernelpkg.__salt__['system.reboot'].assert_not_called()

    def test_upgrade_available_true(self):
        '''
        Test - upgrade_available
        '''
        with patch.object(self._kernelpkg, 'latest_available', return_value=self.KERNEL_LIST[-1]):
            with patch.object(self._kernelpkg, 'latest_installed', return_value=self.KERNEL_LIST[0]):
                self.assertTrue(self._kernelpkg.upgrade_available())

    def test_upgrade_available_false(self):
        '''
        Test - upgrade_available
        '''
        with patch.object(self._kernelpkg, 'latest_available', return_value=self.KERNEL_LIST[-1]):
            with patch.object(self._kernelpkg, 'latest_installed', return_value=self.KERNEL_LIST[-1]):
                self.assertFalse(self._kernelpkg.upgrade_available())

    def test_upgrade_available_inverted(self):
        '''
        Test - upgrade_available
        '''
        with patch.object(self._kernelpkg, 'latest_available', return_value=self.KERNEL_LIST[0]):
            with patch.object(self._kernelpkg, 'latest_installed', return_value=self.KERNEL_LIST[-1]):
                self.assertFalse(self._kernelpkg.upgrade_available())

    def test_remove_active(self):
        '''
        Test - remove kernel package
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stderr': []})
        with patch.dict(self._kernelpkg.__salt__, {'cmd.run_all': mock}):
            with patch.object(self._kernelpkg, 'active', return_value=self.KERNEL_LIST[-1]):
                with patch.object(self._kernelpkg, 'list_installed', return_value=self.KERNEL_LIST):
                    self.assertRaises(CommandExecutionError, self._kernelpkg.remove, release=self.KERNEL_LIST[-1])
                    self._kernelpkg.__salt__['cmd.run_all'].assert_not_called()

    def test_remove_invalid(self):
        '''
        Test - remove kernel package
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stderr': []})
        with patch.dict(self._kernelpkg.__salt__, {'cmd.run_all': mock}):
            with patch.object(self._kernelpkg, 'active', return_value=self.KERNEL_LIST[-1]):
                with patch.object(self._kernelpkg, 'list_installed', return_value=self.KERNEL_LIST):
                    self.assertRaises(CommandExecutionError, self._kernelpkg.remove, release='invalid')
                    self._kernelpkg.__salt__['cmd.run_all'].assert_not_called()



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.unit import skipIf
from tests.support.helpers import destructiveTest

# Import Salt libs
import salt.utils.platform


@skipIf(not salt.utils.platform.is_windows(), 'windows test only')
class WinDNSTest(ModuleCase):
    '''
    Test for salt.modules.win_dns_client
    '''
    @destructiveTest
    def test_add_remove_dns(self):
        '''
        Test add and removing a dns server
        '''
        # Get a list of interfaces on the system
        interfaces = self.run_function('network.interfaces_names')
        skipIf(interfaces.count == 0, 'This test requires a network interface')

        interface = interfaces[0]
        dns = '8.8.8.8'
        # add dns server
        self.assertTrue(self.run_function('win_dns_client.add_dns', [dns, interface], index=42))

        srvs = self.run_function('win_dns_client.get_dns_servers', interface=interface)
        self.assertIn(dns, srvs)

        # remove dns server
        self.assertTrue(self.run_function('win_dns_client.rm_dns', [dns], interface=interface))

        srvs = self.run_function('win_dns_client.get_dns_servers', interface=interface)
        self.assertNotIn(dns, srvs)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)


    tests.integration.states.pip_state
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'''

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import errno
import os
import glob
import shutil
import sys

try:
    import pwd
    HAS_PWD = True
except ImportError:
    HAS_PWD = False

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.helpers import (
    destructiveTest,
    requires_system_grains,
    with_system_user,
    skip_if_not_root,
    with_tempdir
)
from tests.support.mixins import SaltReturnAssertsMixin
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import skipIf

# Import salt libs
import salt.utils.files
import salt.utils.path
import salt.utils.platform
import salt.utils.versions
import salt.utils.win_dacl
import salt.utils.win_functions
import salt.utils.win_runas
from salt.modules.virtualenv_mod import KNOWN_BINARY_NAMES
from salt.exceptions import CommandExecutionError

# Import 3rd-party libs
from salt.ext import six


def can_runas():
    '''
    Detect if we are running in a limited shell (winrm) and are un-able to use
    the runas utility method.
    '''
    if salt.utils.platform.is_windows():
        try:
            salt.utils.win_runas.runas(
                'cmd.exe /c echo 1', 'noexistuser', 'n0existp4ss',
            )
        except WindowsError as exc:  # pylint: disable=undefined-variable
            if exc.winerror == 5:
                # Access Denied
                return False
    return True


CAN_RUNAS = can_runas()


class VirtualEnv(object):
    def __init__(self, test, venv_dir):
        self.venv_dir = venv_dir
        self.test = test

    def __enter__(self):
        ret = self.test.run_function('virtualenv.create', [self.venv_dir])
        self.test.assertEqual(ret['retcode'], 0)

    def __exit__(self, exc_type, exc_value, traceback):
        if os.path.isdir(self.venv_dir):
            shutil.rmtree(self.venv_dir, ignore_errors=True)


@skipIf(salt.utils.path.which_bin(KNOWN_BINARY_NAMES) is None, 'virtualenv not installed')
class PipStateTest(ModuleCase, SaltReturnAssertsMixin):

    @skip_if_not_root
    def test_pip_installed_removed(self):
        '''
        Tests installed and removed states
        '''
        name = 'pudb'
        if name in self.run_function('pip.list'):
            self.skipTest('{0} is already installed, uninstall to run this test'.format(name))
        ret = self.run_state('pip.installed', name=name)
        self.assertSaltTrueReturn(ret)
        ret = self.run_state('pip.removed', name=name)
        self.assertSaltTrueReturn(ret)

    def test_pip_installed_removed_venv(self):
        venv_dir = os.path.join(
            RUNTIME_VARS.TMP, 'pip_installed_removed'
        )
        with VirtualEnv(self, venv_dir):
            name = 'pudb'
            ret = self.run_state('pip.installed', name=name, bin_env=venv_dir)
            self.assertSaltTrueReturn(ret)
            ret = self.run_state('pip.removed', name=name, bin_env=venv_dir)
            self.assertSaltTrueReturn(ret)

    def test_pip_installed_errors(self):
        venv_dir = os.path.join(
            RUNTIME_VARS.TMP, 'pip-installed-errors'
        )
        orig_shell = os.environ.get('SHELL')
        try:
            # Since we don't have the virtualenv created, pip.installed will
            # throw an error.
            # Example error strings:
            #  * "Error installing 'pep8': /tmp/pip-installed-errors: not found"
            #  * "Error installing 'pep8': /bin/sh: 1: /tmp/pip-installed-errors: not found"
            #  * "Error installing 'pep8': /bin/bash: /tmp/pip-installed-errors: No such file or directory"
            os.environ['SHELL'] = '/bin/sh'
            ret = self.run_function('state.sls', mods='pip-installed-errors')
            self.assertSaltFalseReturn(ret)
            self.assertSaltCommentRegexpMatches(
                ret,
                'Error installing \'pep8\':'
            )

            # We now create the missing virtualenv
            ret = self.run_function('virtualenv.create', [venv_dir])
            self.assertEqual(ret['retcode'], 0)

            # The state should not have any issues running now
            ret = self.run_function('state.sls', mods='pip-installed-errors')
            self.assertSaltTrueReturn(ret)
        finally:
            if orig_shell is None:
                # Didn't exist before, don't leave it there. This should never
                # happen, but if it does, we don't want this test to affect
                # others elsewhere in the suite.
                os.environ.pop('SHELL')
            else:
                os.environ['SHELL'] = orig_shell
            if os.path.isdir(venv_dir):
                shutil.rmtree(venv_dir, ignore_errors=True)

    @skipIf(six.PY3, 'Issue is specific to carbon module, which is PY2-only')
    @skipIf(salt.utils.platform.is_windows(), "Carbon does not install in Windows")
    @requires_system_grains
    def test_pip_installed_weird_install(self, grains=None):
        # First, check to see if this is running on CentOS 5 or MacOS.
        # If so, skip this test.
        if grains['os'] in ('CentOS',) and grains['osrelease_info'][0] in (5,):
            self.skipTest('This test does not run reliably on CentOS 5')
        if grains['os'] in ('MacOS',):
            self.skipTest('This test does not run reliably on MacOS')

        ographite = '/opt/graphite'
        if os.path.isdir(ographite):
            self.skipTest(
                'You already have \'{0}\'. This test would overwrite this '
                'directory'.format(ographite)
            )
        try:
            os.makedirs(ographite)
        except OSError as err:
            if err.errno == errno.EACCES:
                # Permission denied
                self.skipTest(
                    'You don\'t have the required permissions to run this test'
                )
        finally:
            if os.path.isdir(ographite):
                shutil.rmtree(ographite, ignore_errors=True)

        venv_dir = os.path.join(RUNTIME_VARS.TMP, 'pip-installed-weird-install')
        try:
            # We may be able to remove this, I had to add it because the custom
            # modules from the test suite weren't available in the jinja
            # context when running the call to state.sls that comes after.
            self.run_function('saltutil.sync_modules')
            # Since we don't have the virtualenv created, pip.installed will
            # throw an error.
            ret = self.run_function(
                'state.sls', mods='pip-installed-weird-install'
            )
            self.assertSaltTrueReturn(ret)

            # We cannot use assertInSaltComment here because we need to skip
            # some of the state return parts
            for key in six.iterkeys(ret):
                self.assertTrue(ret[key]['result'])
                if ret[key]['name'] != 'carbon < 1.1':
                    continue
                self.assertEqual(
                    ret[key]['comment'],
                    'There was no error installing package \'carbon < 1.1\' '
                    'although it does not show when calling \'pip.freeze\'.'
                )
                break
            else:
                raise Exception('Expected state did not run')
        finally:
            if os.path.isdir(ographite):
                shutil.rmtree(ographite, ignore_errors=True)

    def test_issue_2028_pip_installed_state(self):
        ret = self.run_function('state.sls', mods='issue-2028-pip-installed')

        venv_dir = os.path.join(
            RUNTIME_VARS.TMP, 'issue-2028-pip-installed'
        )

        pep8_bin = os.path.join(venv_dir, 'bin', 'pep8')
        if salt.utils.platform.is_windows():
            pep8_bin = os.path.join(venv_dir, 'Scripts', 'pep8.exe')

        try:
            self.assertSaltTrueReturn(ret)
            self.assertTrue(
                os.path.isfile(pep8_bin)
            )
        finally:
            if os.path.isdir(venv_dir):
                shutil.rmtree(venv_dir, ignore_errors=True)

    def test_issue_2087_missing_pip(self):
        venv_dir = os.path.join(
            RUNTIME_VARS.TMP, 'issue-2087-missing-pip'
        )

        try:
            # Let's create the testing virtualenv
            ret = self.run_function('virtualenv.create', [venv_dir])
            self.assertEqual(ret['retcode'], 0)

            # Let's remove the pip binary
            pip_bin = os.path.join(venv_dir, 'bin', 'pip')
            site_dir = self.run_function('virtualenv.get_distribution_path', [venv_dir, 'pip'])
            if salt.utils.platform.is_windows():
                pip_bin = os.path.join(venv_dir, 'Scripts', 'pip.exe')
                site_dir = os.path.join(venv_dir, 'lib', 'site-packages')
            if not os.path.isfile(pip_bin):
                self.skipTest(
                    'Failed to find the pip binary to the test virtualenv'
                )
            os.remove(pip_bin)

            # Also remove the pip dir from site-packages
            # This is needed now that we're using python -m pip instead of the
            # pip binary directly. python -m pip will still work even if the
            # pip binary is missing
            shutil.rmtree(os.path.join(site_dir, 'pip'))

            # Let's run the state which should fail because pip is missing
            ret = self.run_function('state.sls', mods='issue-2087-missing-pip')
            self.assertSaltFalseReturn(ret)
            self.assertInSaltComment(
                'Error installing \'pep8\': Could not find a `pip` binary',
                ret
            )
        finally:
            if os.path.isdir(venv_dir):
                shutil.rmtree(venv_dir, ignore_errors=True)

    def test_issue_5940_multiple_pip_mirrors(self):
        '''
        Test multiple pip mirrors.  This test only works with pip < 7.0.0
        '''
        ret = self.run_function(
            'state.sls', mods='issue-5940-multiple-pip-mirrors'
        )

        venv_dir = os.path.join(
            RUNTIME_VARS.TMP, '5940-multiple-pip-mirrors'
        )

        try:
            self.assertSaltTrueReturn(ret)
            self.assertTrue(
                os.path.isfile(os.path.join(venv_dir, 'bin', 'pep8'))
            )
        except (AssertionError, CommandExecutionError):
            pip_version = self.run_function('pip.version', [venv_dir])
            if salt.utils.versions.compare(ver1=pip_version, oper='>=', ver2='7.0.0'):
                self.skipTest('the --mirrors arg has been deprecated and removed in pip==7.0.0')
        finally:
            if os.path.isdir(venv_dir):
                shutil.rmtree(venv_dir, ignore_errors=True)

    @destructiveTest
    @skip_if_not_root
    @skipIf(not CAN_RUNAS, 'Runas support required')
    @with_system_user('issue-6912', on_existing='delete', delete=True,
                      password='PassWord1!')
    @with_tempdir()
    def test_issue_6912_wrong_owner(self, temp_dir, username):
        # Setup virtual environment directory to be used throughout the test
        venv_dir = os.path.join(temp_dir, '6912-wrong-owner')

        # The virtual environment needs to be in a location that is accessible
        # by both the user running the test and the runas user
        if salt.utils.platform.is_windows():
            salt.utils.win_dacl.set_permissions(temp_dir, username, 'full_control')
        else:
            uid = self.run_function('file.user_to_uid', [username])
            os.chown(temp_dir, uid, -1)

        # Create the virtual environment
        venv_create = self.run_function(
            'virtualenv.create', [venv_dir], user=username,
            password='PassWord1!')
        if venv_create['retcode'] > 0:
            self.skipTest('Failed to create testcase virtual environment: {0}'
                          ''.format(venv_create))

        # pip install passing the package name in `name`
        ret = self.run_state(
            'pip.installed', name='pep8', user=username, bin_env=venv_dir,
            password='PassWord1!')
        self.assertSaltTrueReturn(ret)

        if HAS_PWD:
            uid = pwd.getpwnam(username).pw_uid
        for globmatch in (os.path.join(venv_dir, '**', 'pep8*'),
                          os.path.join(venv_dir, '*', '**', 'pep8*'),
                          os.path.join(venv_dir, '*', '*', '**', 'pep8*')):
            for path in glob.glob(globmatch):
                if HAS_PWD:
                    self.assertEqual(uid, os.stat(path).st_uid)
                elif salt.utils.platform.is_windows():
                    self.assertEqual(
                        salt.utils.win_dacl.get_owner(path), username)

    @destructiveTest
    @skip_if_not_root
    @skipIf(salt.utils.platform.is_darwin(), 'Test is flaky on macosx')
    @skipIf(not CAN_RUNAS, 'Runas support required')
    @with_system_user('issue-6912', on_existing='delete', delete=True,
                      password='PassWord1!')
    @with_tempdir()
    def test_issue_6912_wrong_owner_requirements_file(self, temp_dir, username):
        # Setup virtual environment directory to be used throughout the test
        venv_dir = os.path.join(temp_dir, '6912-wrong-owner')

        # The virtual environment needs to be in a location that is accessible
        # by both the user running the test and the runas user
        if salt.utils.platform.is_windows():
            salt.utils.win_dacl.set_permissions(temp_dir, username, 'full_control')
        else:
            uid = self.run_function('file.user_to_uid', [username])
            os.chown(temp_dir, uid, -1)

        # Create the virtual environment again as it should have been removed
        venv_create = self.run_function(
            'virtualenv.create', [venv_dir], user=username,
            password='PassWord1!')
        if venv_create['retcode'] > 0:
            self.skipTest('failed to create testcase virtual environment: {0}'
                          ''.format(venv_create))

        # pip install using a requirements file
        req_filename = os.path.join(
            RUNTIME_VARS.TMP_STATE_TREE, 'issue-6912-requirements.txt'
        )
        with salt.utils.files.fopen(req_filename, 'wb') as reqf:
            reqf.write(b'pep8\n')

        ret = self.run_state(
            'pip.installed', name='', user=username, bin_env=venv_dir,
            requirements='salt://issue-6912-requirements.txt',
            password='PassWord1!')
        self.assertSaltTrueReturn(ret)

        if HAS_PWD:
            uid = pwd.getpwnam(username).pw_uid
        for globmatch in (os.path.join(venv_dir, '**', 'pep8*'),
                          os.path.join(venv_dir, '*', '**', 'pep8*'),
                          os.path.join(venv_dir, '*', '*', '**', 'pep8*')):
            for path in glob.glob(globmatch):
                if HAS_PWD:
                    self.assertEqual(uid, os.stat(path).st_uid)
                elif salt.utils.platform.is_windows():
                    self.assertEqual(
                        salt.utils.win_dacl.get_owner(path), username)

    def test_issue_6833_pip_upgrade_pip(self):
        # Create the testing virtualenv
        venv_dir = os.path.join(
            RUNTIME_VARS.TMP, '6833-pip-upgrade-pip'
        )
        ret = self.run_function('virtualenv.create', [venv_dir])

        try:
            try:
                self.assertEqual(ret['retcode'], 0)
                self.assertIn(
                    'New python executable',
                    ret['stdout']
                )
            except AssertionError:
                import pprint
                pprint.pprint(ret)
                raise

            # Let's install a fixed version pip over whatever pip was
            # previously installed
            ret = self.run_function(
                'pip.install', ['pip==8.0'], upgrade=True,
                bin_env=venv_dir
            )
            try:
                self.assertEqual(ret['retcode'], 0)
                self.assertIn(
                    'Successfully installed pip',
                    ret['stdout']
                )
            except AssertionError:
                import pprint
                pprint.pprint(ret)
                raise

            # Let's make sure we have pip 8.0 installed
            self.assertEqual(
                self.run_function('pip.list', ['pip'], bin_env=venv_dir),
                {'pip': '8.0.0'}
            )

            # Now the actual pip upgrade pip test
            ret = self.run_state(
                'pip.installed', name='pip==8.0.1', upgrade=True,
                bin_env=venv_dir
            )
            try:
                self.assertSaltTrueReturn(ret)
                self.assertSaltStateChangesEqual(
                    ret, {'pip==8.0.1': 'Installed'})
            except AssertionError:
                import pprint
                pprint.pprint(ret)
                raise
        finally:
            if os.path.isdir(venv_dir):
                shutil.rmtree(venv_dir, ignore_errors=True)

    def test_pip_installed_specific_env(self):
        # Create the testing virtualenv
        venv_dir = os.path.join(
            RUNTIME_VARS.TMP, 'pip-installed-specific-env'
        )

        # Let's write a requirements file
        requirements_file = os.path.join(
            RUNTIME_VARS.TMP_PRODENV_STATE_TREE, 'prod-env-requirements.txt'
        )
        with salt.utils.files.fopen(requirements_file, 'wb') as reqf:
            reqf.write(b'pep8\n')

        try:
            self.run_function('virtualenv.create', [venv_dir])

            # The requirements file should not be found the base environment
            ret = self.run_state(
                'pip.installed', name='', bin_env=venv_dir,
                requirements='salt://prod-env-requirements.txt'
            )
            self.assertSaltFalseReturn(ret)
            self.assertInSaltComment(
                "'salt://prod-env-requirements.txt' not found", ret
            )

            # The requirements file must be found in the prod environment
            ret = self.run_state(
                'pip.installed', name='', bin_env=venv_dir, saltenv='prod',
                requirements='salt://prod-env-requirements.txt'
            )
            self.assertSaltTrueReturn(ret)
            self.assertInSaltComment(
                'Successfully processed requirements file '
                'salt://prod-env-requirements.txt', ret
            )

            # We're using the base environment but we're passing the prod
            # environment as an url arg to salt://
            ret = self.run_state(
                'pip.installed', name='', bin_env=venv_dir,
                requirements='salt://prod-env-requirements.txt?saltenv=prod'
            )
            self.assertSaltTrueReturn(ret)
            self.assertInSaltComment(
                'Requirements were already installed.',
                ret
            )
        finally:
            if os.path.isdir(venv_dir):
                shutil.rmtree(venv_dir, ignore_errors=True)
            if os.path.isfile(requirements_file):
                os.unlink(requirements_file)

    def test_22359_pip_installed_unless_does_not_trigger_warnings(self):
        # This test case should be moved to a format_call unit test specific to
        # the state internal keywords
        venv_dir = os.path.join(RUNTIME_VARS.TMP, 'pip-installed-unless')
        venv_create = self.run_function('virtualenv.create', [venv_dir])
        if venv_create['retcode'] > 0:
            self.skipTest(
                'Failed to create testcase virtual environment: {0}'.format(
                    venv_create
                )
            )

        false_cmd = '/bin/false'
        if salt.utils.platform.is_windows():
            false_cmd = 'exit 1 >nul'
        try:
            ret = self.run_state(
                'pip.installed', name='pep8', bin_env=venv_dir, unless=false_cmd
            )
            self.assertSaltTrueReturn(ret)
            self.assertNotIn('warnings', next(six.itervalues(ret)))
        finally:
            if os.path.isdir(venv_dir):
                shutil.rmtree(venv_dir, ignore_errors=True)

    @skipIf(sys.version_info[:2] >= (3, 6), 'Old version of virtualenv too old for python3.6')
    @skipIf(salt.utils.platform.is_windows(), "Carbon does not install in Windows")
    def test_46127_pip_env_vars(self):
        '''
        Test that checks if env_vars passed to pip.installed are also passed
        to pip.freeze while checking for existing installations
        '''
        # This issue is most easily checked while installing carbon
        # Much of the code here comes from the test_weird_install function above
        ographite = '/opt/graphite'
        if os.path.isdir(ographite):
            self.skipTest(
                'You already have \'{0}\'. This test would overwrite this '
                'directory'.format(ographite)
            )
        try:
            os.makedirs(ographite)
        except OSError as err:
            if err.errno == errno.EACCES:
                # Permission denied
                self.skipTest(
                    'You don\'t have the required permissions to run this test'
                )
        finally:
            if os.path.isdir(ographite):
                shutil.rmtree(ographite, ignore_errors=True)

        venv_dir = os.path.join(RUNTIME_VARS.TMP, 'issue-46127-pip-env-vars')
        try:
            # We may be able to remove this, I had to add it because the custom
            # modules from the test suite weren't available in the jinja
            # context when running the call to state.sls that comes after.
            self.run_function('saltutil.sync_modules')
            # Since we don't have the virtualenv created, pip.installed will
            # throw an error.
            ret = self.run_function(
                'state.sls', mods='issue-46127-pip-env-vars'
            )
            self.assertSaltTrueReturn(ret)
            for key in six.iterkeys(ret):
                self.assertTrue(ret[key]['result'])
                if ret[key]['name'] != 'carbon < 1.3':
                    continue
                self.assertEqual(
                    ret[key]['comment'],
                    'All packages were successfully installed'
                )
                break
            else:
                raise Exception('Expected state did not run')
            # Run the state again. Now the already installed message should
            # appear
            ret = self.run_function(
                'state.sls', mods='issue-46127-pip-env-vars'
            )
            self.assertSaltTrueReturn(ret)
            # We cannot use assertInSaltComment here because we need to skip
            # some of the state return parts
            for key in six.iterkeys(ret):
                self.assertTrue(ret[key]['result'])
                # As we are re-running the formula, some states will not be run
                # and "name" may or may not be present, so we use .get() pattern
                if ret[key].get('name', '') != 'carbon < 1.3':
                    continue
                self.assertEqual(
                    ret[key]['comment'],
                    ('All packages were successfully installed'))
                break
            else:
                raise Exception('Expected state did not run')
        finally:
            if os.path.isdir(ographite):
                shutil.rmtree(ographite, ignore_errors=True)
            if os.path.isdir(venv_dir):
                shutil.rmtree(venv_dir)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)


    integration.loader.ext_modules
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Test Salt's loader regarding external overrides
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import time

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.paths import TMP


class LoaderOverridesTest(ModuleCase):

    def setUp(self):
        self.run_function('saltutil.sync_modules')

    def test_overridden_internal(self):
        # To avoid a race condition on Windows, we need to make sure the
        # `override_test.py` file is present in the _modules directory before
        # trying to list all functions. This test may execute before the
        # minion has finished syncing down the files it needs.
        module = os.path.join(TMP, 'rootdir', 'cache', 'files',
                              'base', '_modules', 'override_test.py')
        tries = 0
        while not os.path.exists(module):
            tries += 1
            if tries > 60:
                break
            time.sleep(1)

        funcs = self.run_function('sys.list_functions')

        # We placed a test module under _modules.
        # The previous functions should also still exist.
        self.assertIn('test.ping', funcs)

        # A non existing function should, of course, not exist
        self.assertNotIn('brain.left_hemisphere', funcs)

        # There should be a new function for the test module, recho
        self.assertIn('test.recho', funcs)

        text = 'foo bar baz quo qux'
        self.assertEqual(
            self.run_function('test.echo', arg=[text])[::-1],
            self.run_function('test.recho', arg=[text]),
        )



# -*- coding: utf-8 -*-
'''
Tests for existence of manpages
'''
# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import shutil

# Import Salt libs
import salt.utils.platform

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.paths import TMP
from tests.support.unit import skipIf


@skipIf(salt.utils.platform.is_windows(), 'minion is windows')
class ManTest(ModuleCase):
    rootdir = os.path.join(TMP, 'mantest')
    # Map filenames to search strings which should be in the manpage
    manpages = {
        'salt-cp.1': [
            'salt-cp Documentation',
            'copies files from the master',
        ],
        'salt-cloud.1': [
            'Salt Cloud Command',
            'Provision virtual machines in the cloud',
        ],
        'salt-call.1': [
            'salt-call Documentation',
            'run module functions locally',
        ],
        'salt-api.1': [
            'salt-api Command',
            'Start interfaces used to remotely connect',
        ],
        'salt-unity.1': [
            'salt-unity Command',
            'unified invocation wrapper',
        ],
        'salt-syndic.1': [
            'salt-syndic Documentation',
            'Salt syndic daemon',
        ],
        'salt-ssh.1': [
            'salt-ssh Documentation',
            'executed using only SSH',
        ],
        'salt-run.1': [
            'salt-run Documentation',
            'frontend command for executing',
        ],
        'salt-proxy.1': [
            'salt-proxy Documentation',
            'proxies these commands',
        ],
        'salt-minion.1': [
            'salt-minion Documentation',
            'Salt minion daemon',
        ],
        'salt-master.1': [
            'salt-master Documentation',
            'Salt master daemon',
        ],
        'salt-key.1': [
            'salt-key Documentation',
            'management of Salt server public keys',
        ],
        'salt.1': [
            'allows for commands to be executed',
        ],
        'salt.7': [
            'Salt Documentation',
        ],
        'spm.1': [
            'Salt Package Manager Command',
            'command for managing Salt packages',
        ],
    }

    def setUp(self):
        if not self.run_function('mantest.install', [self.rootdir]):
            self.fail('Failed to install salt to {0}'.format(self.rootdir))

    @classmethod
    def tearDownClass(cls):
        try:
            shutil.rmtree(cls.rootdir)
        except OSError:
            pass

    def test_man(self):
        '''
        Make sure that man pages are installed
        '''
        ret = self.run_function('mantest.search', [self.manpages, self.rootdir])
        # The above function returns True if successful and an exception (which
        # will manifest in the return as a stringified exception) if
        # unsuccessful. Therefore, a simple assertTrue is not sufficient.
        if ret is not True:
            self.fail(ret)



# -*- coding: utf-8 -*-
'''
    :copyright: Copyright 2013-2017 by the SaltStack Team, see AUTHORS for more details.
    :license: Apache 2.0, see LICENSE for more details.


    tests.support.win_installer
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Fetches the binary Windows installer
'''
from __future__ import absolute_import
import hashlib
import requests
import re

PREFIX = 'Salt-Minion-'
REPO = "https://repo.saltstack.com/windows"


def iter_installers(content):
    '''
    Parse a list of windows installer links and their corresponding md5
    checksum links.
    '''
    HREF_RE = "<a href=\"(.*?)\">"
    installer, md5 = None, None
    for m in re.finditer(HREF_RE, content):
        x = m.groups()[0]
        if not x.startswith(PREFIX):
            continue
        if x.endswith(('zip', 'sha256')):
            continue
        if installer:
            if x != installer + '.md5':
                raise Exception("Unable to parse response")
            md5 = x
            yield installer, md5
            installer, md5 = None, None
        else:
            installer = x


def split_installer(name):
    '''
    Return a tuple of the salt version, python verison and architecture from an
    installer name.
    '''
    x = name[len(PREFIX):]
    return x.split('-')[:3]


def latest_version(repo=REPO):
    '''
    Return the latest version found on the salt repository webpage.
    '''
    content = requests.get(repo).content.decode('utf-8')
    for name, md5 in iter_installers(content):
        pass
    return split_installer(name)[0]


def installer_name(salt_ver, py_ver='Py2', arch='AMD64'):
    '''
    Create an installer file name
    '''
    return "Salt-Minion-{}-{}-{}-Setup.exe".format(salt_ver, py_ver, arch)


def latest_installer_name(repo=REPO, **kwargs):
    '''
    Fetch the latest installer name
    '''
    return installer_name(latest_version(repo), **kwargs)


def download_and_verify(fp, name, repo=REPO):
    '''
    Download an installer and verify it's contents.
    '''
    md5 = "{}.md5".format(name)
    url = lambda x: "{}/{}".format(repo, x)
    resp = requests.get(url(md5))
    if resp.status_code != 200:
        raise Exception("Unable to fetch installer md5")
    installer_md5 = resp.text.strip().split()[0].lower()
    resp = requests.get(url(name), stream=True)
    if resp.status_code != 200:
        raise Exception("Unable to fetch installer")
    md5hsh = hashlib.md5()
    for chunk in resp.iter_content(chunk_size=1024):
        md5hsh.update(chunk)
        fp.write(chunk)
    if md5hsh.hexdigest() != installer_md5:
        raise Exception("Installer's hash does not match {} != {}".format(
            md5hsh.hexdigest(), installer_md5
        ))



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase
from tests.support.mock import (
    MagicMock,
    mock_open,
    patch,
)
# Import Salt Libs
import salt.modules.hosts as hosts
import salt.utils.data
import salt.utils.platform
import salt.utils.stringutils
from salt.ext.six.moves import StringIO
from salt.ext import six


class HostsTestCase(TestCase, LoaderModuleMockMixin):
    '''
    TestCase for salt.modules.hosts
    '''
    def setup_loader_modules(self):
        return {hosts: {}}

    # 'list_hosts' function tests: 1

    def test_list_hosts(self):
        '''
        Tests return the hosts found in the hosts file
        '''
        with patch('salt.modules.hosts._list_hosts',
                   MagicMock(return_value={'10.10.10.10': ['Salt1', 'Salt2']})):
            self.assertDictEqual({'10.10.10.10': ['Salt1', 'Salt2']},
                                 hosts.list_hosts())

    # 'get_ip' function tests: 3

    def test_get_ip(self):
        '''
        Tests return ip associated with the named host
        '''
        with patch('salt.modules.hosts._list_hosts',
                   MagicMock(return_value={'10.10.10.10': ['Salt1', 'Salt2']})):
            self.assertEqual('10.10.10.10', hosts.get_ip('Salt1'))

            self.assertEqual('', hosts.get_ip('Salt3'))

    def test_get_ip_none(self):
        '''
        Tests return ip associated with the named host
        '''
        with patch('salt.modules.hosts._list_hosts', MagicMock(return_value='')):
            self.assertEqual('', hosts.get_ip('Salt1'))

    # 'get_alias' function tests: 2

    def test_get_alias(self):
        '''
        Tests return the list of aliases associated with an ip
        '''
        with patch('salt.modules.hosts._list_hosts',
                   MagicMock(return_value={'10.10.10.10': ['Salt1', 'Salt2']})):
            self.assertListEqual(['Salt1', 'Salt2'], hosts.get_alias('10.10.10.10'))

    def test_get_alias_none(self):
        '''
        Tests return the list of aliases associated with an ip
        '''
        with patch('salt.modules.hosts._list_hosts',
                   MagicMock(return_value={'10.10.10.10': ['Salt1', 'Salt2']})):
            self.assertListEqual([], hosts.get_alias('10.10.10.11'))

    # 'has_pair' function tests: 1

    def test_has_pair(self):
        '''
        Tests return True / False if the alias is set
        '''
        with patch('salt.modules.hosts._list_hosts',
                   MagicMock(return_value={'10.10.10.10': ['Salt1', 'Salt2']})):
            self.assertTrue(hosts.has_pair('10.10.10.10', 'Salt1'))

            self.assertFalse(hosts.has_pair('10.10.10.10', 'Salt3'))

    # 'set_host' function tests: 3

    def test_set_host(self):
        '''
        Tests true if the alias is set
        '''
        hosts_file = '/etc/hosts'
        if salt.utils.platform.is_windows():
            hosts_file = r'C:\Windows\System32\Drivers\etc\hosts'

        with patch('salt.modules.hosts.__get_hosts_filename',
                  MagicMock(return_value=hosts_file)), \
                patch('os.path.isfile', MagicMock(return_value=False)), \
                    patch.dict(hosts.__salt__,
                               {'config.option': MagicMock(return_value=None)}):
            self.assertFalse(hosts.set_host('10.10.10.10', 'Salt1'))

    def test_set_host_true(self):
        '''
        Tests true if the alias is set
        '''
        with patch('salt.modules.hosts.__get_hosts_filename',
                   MagicMock(return_value='/etc/hosts')), \
                patch('os.path.isfile', MagicMock(return_value=True)), \
                    patch('salt.utils.files.fopen', mock_open(b'')):
            mock_opt = MagicMock(return_value=None)
            with patch.dict(hosts.__salt__, {'config.option': mock_opt}):
                self.assertTrue(hosts.set_host('10.10.10.10', 'Salt1'))

    def test_set_host_true_remove(self):
        '''
        Test if an empty hosts value removes existing entries
        '''
        with patch('salt.modules.hosts.__get_hosts_filename',
                   MagicMock(return_value='/etc/hosts')), \
                patch('os.path.isfile', MagicMock(return_value=True)):
            data = ['\n'.join((
                '1.1.1.1 foo.foofoo foo',
                '2.2.2.2 bar.barbar bar',
                '3.3.3.3 asdf.asdfadsf asdf',
                '1.1.1.1 foofoo.foofoo foofoo',
            ))]

            class TmpStringIO(StringIO, object):
                def __init__(self, fn, mode='r'):
                    self.mode = mode
                    initial_value = data[0]
                    if 'w' in self.mode:
                        initial_value = ''
                    super(TmpStringIO, self).__init__(initial_value)

                def __enter__(self):
                    return self

                def __exit__(self, exc_type, exc_value, traceback):
                    self.close()

                def close(self):
                    # Don't save unless there's something there. In Windows
                    # the class gets initialized the first time with mode = w
                    # which sets the initial value to ''. When the class closes
                    # it clears out data and causes the test to fail.
                    # I don't know why it get's initialized with a mode of 'w'
                    # For the purposes of this test data shouldn't be empty
                    # This is a problem with this class and not with the hosts
                    # module
                    if self.getvalue():
                        data[0] = self.getvalue()
                    StringIO.close(self)

                def read(self, *args):
                    ret = super(TmpStringIO, self).read(*args)
                    if six.PY3 and 'b' in self.mode:
                        return salt.utils.stringutils.to_bytes(ret)
                    else:
                        return ret

                def write(self, s, *args):
                    if six.PY3:
                        if 'b' in self.mode:
                            if not isinstance(s, bytes):
                                # Make this act like a binary filehandle
                                raise TypeError("a bytes-like object is required, not 'str'")
                            # The StringIO wants a str type, it won't take
                            # bytes. Convert before writing to it.
                            return super(TmpStringIO, self).write(
                                salt.utils.stringutils.to_str(s), *args)
                        else:
                            if not isinstance(s, str):
                                # Make this act like a non-binary filehandle
                                raise TypeError("write() argument must be str, not bytes")
                    return super(TmpStringIO, self).write(s, *args)

                def readlines(self):
                    ret = super(TmpStringIO, self).readlines()
                    if six.PY3 and 'b' in self.mode:
                        return salt.utils.data.encode(ret)
                    else:
                        return ret

                def writelines(self, lines):
                    for line in lines:
                        self.write(line)

            expected = '\n'.join((
                '2.2.2.2 bar.barbar bar',
                '3.3.3.3 asdf.asdfadsf asdf',
            )) + '\n'

            with patch('salt.utils.files.fopen', TmpStringIO):
                mock_opt = MagicMock(return_value=None)
                with patch.dict(hosts.__salt__, {'config.option': mock_opt}):
                    self.assertTrue(hosts.set_host('1.1.1.1', ' '))

            self.assertEqual(data[0], expected)

    # 'rm_host' function tests: 2

    def test_rm_host(self):
        '''
        Tests if specified host entry gets removed from the hosts file
        '''
        with patch('salt.utils.files.fopen', mock_open(b'')), \
                patch('salt.modules.hosts.__get_hosts_filename',
                      MagicMock(return_value='/etc/hosts')), \
                patch('salt.modules.hosts.has_pair',
                      MagicMock(return_value=True)), \
                patch('os.path.isfile', MagicMock(return_value=True)):
            mock_opt = MagicMock(return_value=None)
            with patch.dict(hosts.__salt__, {'config.option': mock_opt}):
                self.assertTrue(hosts.rm_host('10.10.10.10', 'Salt1'))

    def test_rm_host_false(self):
        '''
        Tests if specified host entry gets removed from the hosts file
        '''
        with patch('salt.modules.hosts.has_pair', MagicMock(return_value=False)):
            self.assertTrue(hosts.rm_host('10.10.10.10', 'Salt1'))

    # 'add_host' function tests: 3

    def test_add_host(self):
        '''
        Tests if specified host entry gets added from the hosts file
        '''
        hosts_file = '/etc/hosts'
        if salt.utils.platform.is_windows():
            hosts_file = r'C:\Windows\System32\Drivers\etc\hosts'

        with patch('salt.utils.files.fopen', mock_open()), \
                patch('salt.modules.hosts.__get_hosts_filename',
                      MagicMock(return_value=hosts_file)):
            mock_opt = MagicMock(return_value=None)
            with patch.dict(hosts.__salt__, {'config.option': mock_opt}):
                self.assertTrue(hosts.add_host('10.10.10.10', 'Salt1'))

    def test_add_host_no_file(self):
        '''
        Tests if specified host entry gets added from the hosts file
        '''
        with patch('salt.utils.files.fopen', mock_open()), \
                patch('os.path.isfile', MagicMock(return_value=False)):
            mock_opt = MagicMock(return_value=None)
            with patch.dict(hosts.__salt__, {'config.option': mock_opt}):
                self.assertFalse(hosts.add_host('10.10.10.10', 'Salt1'))

    def test_add_host_create_entry(self):
        '''
        Tests if specified host entry gets added from the hosts file
        '''
        with patch('salt.utils.files.fopen', mock_open()), \
                patch('os.path.isfile', MagicMock(return_value=True)):
            mock_opt = MagicMock(return_value=None)
            with patch.dict(hosts.__salt__, {'config.option': mock_opt}):
                self.assertTrue(hosts.add_host('10.10.10.10', 'Salt1'))



# -*- coding: utf-8 -*-
'''
Tests for salt.utils.dateutils
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import datetime

# Import Salt Testing libs
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt libs
import salt.utils.dateutils

# Import 3rd-party libs
try:
    import timelib  # pylint: disable=import-error,unused-import
    HAS_TIMELIB = True
except ImportError:
    HAS_TIMELIB = False


class DateutilsTestCase(TestCase):

    @skipIf(NO_MOCK, NO_MOCK_REASON)
    def test_date_cast(self):
        now = datetime.datetime.now()
        with patch('datetime.datetime'):
            datetime.datetime.now.return_value = now
            self.assertEqual(now, salt.utils.dateutils.date_cast(None))
        self.assertEqual(now, salt.utils.dateutils.date_cast(now))
        try:
            ret = salt.utils.dateutils.date_cast('Mon Dec 23 10:19:15 MST 2013')
            expected_ret = datetime.datetime(2013, 12, 23, 10, 19, 15)
            self.assertEqual(ret, expected_ret)
        except RuntimeError:
            if not HAS_TIMELIB:
                # Unparseable without timelib installed
                self.skipTest('\'timelib\' is not installed')
            else:
                raise

    @skipIf(not HAS_TIMELIB, '\'timelib\' is not installed')
    def test_strftime(self):

        # Taken from doctests

        expected_ret = '2002-12-25'

        src = datetime.datetime(2002, 12, 25, 12, 00, 00, 00)
        ret = salt.utils.dateutils.strftime(src)
        self.assertEqual(ret, expected_ret)

        src = '2002/12/25'
        ret = salt.utils.dateutils.strftime(src)
        self.assertEqual(ret, expected_ret)

        src = 1040814000
        ret = salt.utils.dateutils.strftime(src)
        self.assertEqual(ret, expected_ret)

        src = '1040814000'
        ret = salt.utils.dateutils.strftime(src)
        self.assertEqual(ret, expected_ret)



# -*- coding: utf-8 -*-
# pylint: disable=unused-argument

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, MagicMock, patch

# Import Salt libs
import salt.states.jboss7 as jboss7
from salt.exceptions import CommandExecutionError
from salt.ext import six


@skipIf(NO_MOCK, NO_MOCK_REASON)
class JBoss7StateTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        return {
            jboss7: {
                '__salt__': {
                    'jboss7.read_datasource': MagicMock(),
                    'jboss7.create_datasource': MagicMock(),
                    'jboss7.update_datasource': MagicMock(),
                    'jboss7.remove_datasource': MagicMock(),
                    'jboss7.read_simple_binding': MagicMock(),
                    'jboss7.create_simple_binding': MagicMock(),
                    'jboss7.update_simple_binding': MagicMock(),
                }
            }
        }

    def test_should_create_new_datasource_if_not_exists(self):
        # given
        datasource_properties = {'connection-url': 'jdbc:/old-connection-url'}
        ds_status = {'created': False}

        def read_func(jboss_config, name, profile):
            if ds_status['created']:
                return {'success': True, 'result': datasource_properties}
            else:
                return {'success': False, 'err_code': 'JBAS014807'}

        def create_func(jboss_config, name, datasource_properties, profile):
            ds_status['created'] = True
            return {'success': True}

        read_mock = MagicMock(side_effect=read_func)
        create_mock = MagicMock(side_effect=create_func)
        update_mock = MagicMock()
        with patch.dict(jboss7.__salt__, {'jboss7.read_datasource': read_mock,
                                          'jboss7.create_datasource': create_mock,
                                          'jboss7.update_datasource': update_mock}):

            # when
            result = jboss7.datasource_exists(name='appDS', jboss_config={},
                                              datasource_properties=datasource_properties, profile=None)

            # then
            create_mock.assert_called_with(name='appDS', jboss_config={},
                                           datasource_properties=datasource_properties, profile=None)

            self.assertFalse(update_mock.called)
            self.assertEqual(result['comment'], 'Datasource created.')

    def test_should_update_the_datasource_if_exists(self):
        ds_status = {'updated': False}

        def read_func(jboss_config, name, profile):
            if ds_status['updated']:
                return {'success': True, 'result': {'connection-url': 'jdbc:/new-connection-url'}}
            else:
                return {'success': True, 'result': {'connection-url': 'jdbc:/old-connection-url'}}

        def update_func(jboss_config, name, new_properties, profile):
            ds_status['updated'] = True
            return {'success': True}

        read_mock = MagicMock(side_effect=read_func)
        create_mock = MagicMock()
        update_mock = MagicMock(side_effect=update_func)
        with patch.dict(jboss7.__salt__, {'jboss7.read_datasource': read_mock,
                                          'jboss7.create_datasource': create_mock,
                                          'jboss7.update_datasource': update_mock}):
            result = jboss7.datasource_exists(name='appDS', jboss_config={},
                                              datasource_properties={'connection-url': 'jdbc:/new-connection-url'},
                                              profile=None)

            update_mock.assert_called_with(name='appDS', jboss_config={},
                                           new_properties={'connection-url': 'jdbc:/new-connection-url'},
                                           profile=None)
            self.assertTrue(read_mock.called)
            self.assertEqual(result['comment'], 'Datasource updated.')

    def test_should_recreate_the_datasource_if_specified(self):
        read_mock = MagicMock(return_value={'success': True,
                                            'result': {'connection-url': 'jdbc:/same-connection-url'}})
        create_mock = MagicMock(return_value={'success': True})
        remove_mock = MagicMock(return_value={'success': True})
        update_mock = MagicMock()
        with patch.dict(jboss7.__salt__, {'jboss7.read_datasource': read_mock,
                                          'jboss7.create_datasource': create_mock,
                                          'jboss7.remove_datasource': remove_mock,
                                          'jboss7.update_datasource': update_mock}):

            result = jboss7.datasource_exists(name='appDS', jboss_config={},
                                              datasource_properties={'connection-url': 'jdbc:/same-connection-url'},
                                              recreate=True)

            remove_mock.assert_called_with(name='appDS', jboss_config={}, profile=None)
            create_mock.assert_called_with(name='appDS', jboss_config={},
                                           datasource_properties={'connection-url': 'jdbc:/same-connection-url'},
                                           profile=None)
            self.assertEqual(result['changes']['removed'], 'appDS')
            self.assertEqual(result['changes']['created'], 'appDS')

    def test_should_inform_if_the_datasource_has_not_changed(self):
        read_mock = MagicMock(return_value={'success': True,
                                            'result': {'connection-url': 'jdbc:/same-connection-url'}})
        create_mock = MagicMock()
        remove_mock = MagicMock()
        update_mock = MagicMock(return_value={'success': True})

        with patch.dict(jboss7.__salt__, {'jboss7.read_datasource': read_mock,
                                          'jboss7.create_datasource': create_mock,
                                          'jboss7.remove_datasource': remove_mock,
                                          'jboss7.update_datasource': update_mock}):

            result = jboss7.datasource_exists(name='appDS', jboss_config={},
                                              datasource_properties={'connection-url': 'jdbc:/old-connection-url'})

            update_mock.assert_called_with(name='appDS', jboss_config={},
                                           new_properties={'connection-url': 'jdbc:/old-connection-url'}, profile=None)
            self.assertFalse(create_mock.called)
            self.assertEqual(result['comment'], 'Datasource not changed.')

    def test_should_create_binding_if_not_exists(self):
        # given
        binding_status = {'created': False}

        def read_func(jboss_config, binding_name, profile):
            if binding_status['created']:
                return {'success': True, 'result': {'value': 'DEV'}}
            else:
                return {'success': False, 'err_code': 'JBAS014807'}

        def create_func(jboss_config, binding_name, value, profile):
            binding_status['created'] = True
            return {'success': True}

        read_mock = MagicMock(side_effect=read_func)
        create_mock = MagicMock(side_effect=create_func)
        update_mock = MagicMock()

        with patch.dict(jboss7.__salt__, {'jboss7.read_simple_binding': read_mock,
                                          'jboss7.create_simple_binding': create_mock,
                                          'jboss7.update_simple_binding': update_mock}):

            # when
            result = jboss7.bindings_exist(name='bindings', jboss_config={}, bindings={'env': 'DEV'}, profile=None)

            # then
            create_mock.assert_called_with(jboss_config={}, binding_name='env', value='DEV', profile=None)
            self.assertEqual(update_mock.call_count, 0)
            self.assertEqual(result['changes'], {'added': 'env:DEV\n'})
            self.assertEqual(result['comment'], 'Bindings changed.')

    def test_should_update_bindings_if_exists_and_different(self):
        # given
        binding_status = {'updated': False}

        def read_func(jboss_config, binding_name, profile):
            if binding_status['updated']:
                return {'success': True, 'result': {'value': 'DEV2'}}
            else:
                return {'success': True, 'result': {'value': 'DEV'}}

        def update_func(jboss_config, binding_name, value, profile):
            binding_status['updated'] = True
            return {'success': True}

        read_mock = MagicMock(side_effect=read_func)
        create_mock = MagicMock()
        update_mock = MagicMock(side_effect=update_func)

        with patch.dict(jboss7.__salt__, {'jboss7.read_simple_binding': read_mock,
                                          'jboss7.create_simple_binding': create_mock,
                                          'jboss7.update_simple_binding': update_mock}):
            # when
            result = jboss7.bindings_exist(name='bindings', jboss_config={}, bindings={'env': 'DEV2'}, profile=None)

            # then
            update_mock.assert_called_with(jboss_config={}, binding_name='env', value='DEV2', profile=None)
            self.assertEqual(create_mock.call_count, 0)
            self.assertEqual(result['changes'], {'changed': 'env:DEV->DEV2\n'})
            self.assertEqual(result['comment'], 'Bindings changed.')

    def test_should_not_update_bindings_if_same(self):
        # given
        read_mock = MagicMock(return_value={'success': True, 'result': {'value': 'DEV2'}})
        create_mock = MagicMock()
        update_mock = MagicMock()

        with patch.dict(jboss7.__salt__, {'jboss7.read_simple_binding': read_mock,
                                          'jboss7.create_simple_binding': create_mock,
                                          'jboss7.update_simple_binding': update_mock}):
            # when
            result = jboss7.bindings_exist(name='bindings', jboss_config={}, bindings={'env': 'DEV2'})

            # then
            self.assertEqual(create_mock.call_count, 0)
            self.assertEqual(update_mock.call_count, 0)
            self.assertEqual(result['changes'], {})
            self.assertEqual(result['comment'], 'Bindings not changed.')

    def test_should_raise_exception_if_cannot_create_binding(self):
        def read_func(jboss_config, binding_name, profile):
            return {'success': False, 'err_code': 'JBAS014807'}

        def create_func(jboss_config, binding_name, value, profile):
            return {'success': False, 'failure-description': 'Incorrect binding name.'}

        read_mock = MagicMock(side_effect=read_func)
        create_mock = MagicMock(side_effect=create_func)
        update_mock = MagicMock()

        with patch.dict(jboss7.__salt__, {'jboss7.read_simple_binding': read_mock,
                                          'jboss7.create_simple_binding': create_mock,
                                          'jboss7.update_simple_binding': update_mock}):
            # when
            try:
                jboss7.bindings_exist(name='bindings', jboss_config={}, bindings={'env': 'DEV2'}, profile=None)
                self.fail('An exception should be thrown')
            except CommandExecutionError as e:
                self.assertEqual(six.text_type(e), 'Incorrect binding name.')

    def test_should_raise_exception_if_cannot_update_binding(self):
        def read_func(jboss_config, binding_name, profile):
            return {'success': True, 'result': {'value': 'DEV'}}

        def update_func(jboss_config, binding_name, value, profile):
            return {'success': False, 'failure-description': 'Incorrect binding name.'}

        read_mock = MagicMock(side_effect=read_func)
        create_mock = MagicMock()
        update_mock = MagicMock(side_effect=update_func)

        with patch.dict(jboss7.__salt__, {'jboss7.read_simple_binding': read_mock,
                                          'jboss7.create_simple_binding': create_mock,
                                          'jboss7.update_simple_binding': update_mock}):

            # when
            try:
                jboss7.bindings_exist(name='bindings', jboss_config={},
                                      bindings={'env': '!@#!///some weird value'},
                                      profile=None)
                self.fail('An exception should be thrown')
            except CommandExecutionError as e:
                self.assertEqual(six.text_type(e), 'Incorrect binding name.')



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''
# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)

# Import Salt Libs
import salt.states.portage_config as portage_config


@skipIf(NO_MOCK, NO_MOCK_REASON)
class PortageConfigTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.states.portage_config
    '''
    def setup_loader_modules(self):
        return {portage_config: {'__opts__': {'test': True}}}

    # 'mod_init' function tests: 1

    def test_mod_init(self):
        '''
        Test to enforce a nice structure on the configuration files.
        '''
        name = 'salt'

        mock = MagicMock(side_effect=[True, Exception])
        with patch.dict(portage_config.__salt__,
                        {'portage_config.enforce_nice_config': mock}):
            self.assertTrue(portage_config.mod_init(name))

            self.assertFalse(portage_config.mod_init(name))

    # 'flags' function tests: 1

    def test_flags(self):
        '''
        Test to enforce the given flags on the given package or ``DEPEND`` atom.
        '''
        with patch('traceback.format_exc', MagicMock(return_value='SALTSTACK')):
            name = 'salt'

            ret = {'name': name,
                   'result': False,
                   'comment': 'SALTSTACK',
                   'changes': {}}

            mock = MagicMock(side_effect=Exception('error'))
            with patch.dict(portage_config.__salt__,
                            {'portage_config.get_missing_flags': mock}):
                self.assertDictEqual(portage_config.flags(name, use='openssl'), ret)

                self.assertDictEqual(portage_config.flags(name,
                                                          accept_keywords=True),
                                     ret)

                self.assertDictEqual(portage_config.flags(name, env=True), ret)

                self.assertDictEqual(portage_config.flags(name, license=True), ret)

                self.assertDictEqual(portage_config.flags(name, properties=True),
                                     ret)

                self.assertDictEqual(portage_config.flags(name, mask=True), ret)

                self.assertDictEqual(portage_config.flags(name, unmask=True), ret)

                ret.update({'comment': '', 'result': True})
                self.assertDictEqual(portage_config.flags(name), ret)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import
import os
import os.path
import tempfile

# Import Salt Testing libs
from tests.support.unit import TestCase

# Import Salt libs
import salt.loader
import salt.config
import tests.integration as integration
from salt.exceptions import SaltRenderError
from salt.ext.six.moves import StringIO

# Import 3rd-party libs
from salt.ext import six


REQUISITES = ['require', 'require_in', 'use', 'use_in', 'watch', 'watch_in']


class StateConfigRendererTestCase(TestCase):

    def setUp(self):
        self.root_dir = tempfile.mkdtemp(dir=integration.TMP)
        self.state_tree_dir = os.path.join(self.root_dir, 'state_tree')
        self.cache_dir = os.path.join(self.root_dir, 'cachedir')
        if not os.path.isdir(self.root_dir):
            os.makedirs(self.root_dir)

        if not os.path.isdir(self.state_tree_dir):
            os.makedirs(self.state_tree_dir)

        if not os.path.isdir(self.cache_dir):
            os.makedirs(self.cache_dir)
        self.config = salt.config.minion_config(None)
        self.config['root_dir'] = self.root_dir
        self.config['state_events'] = False
        self.config['id'] = 'match'
        self.config['file_client'] = 'local'
        self.config['file_roots'] = dict(base=[self.state_tree_dir])
        self.config['cachedir'] = self.cache_dir
        self.config['test'] = False
        self._renderers = salt.loader.render(
            self.config,
            {'config.get': lambda a, b: False}
        )

    def tearDown(self):
        for attrname in ('config', '_renderers'):
            try:
                delattr(self, attrname)
            except AttributeError:
                continue

    def _render_sls(self,
                    content,
                    sls='',
                    saltenv='base',
                    argline='-G yaml . jinja',
                    **kws):
        return self._renderers['stateconf'](
            StringIO(content), saltenv=saltenv, sls=sls,
            argline=argline,
            renderers=salt.loader.render(self.config, {}),
            **kws
        )

    def test_state_config(self):
        result = self._render_sls('''
.sls_params:
  stateconf.set:
    - name1: value1
    - name2: value2

.extra:
  stateconf:
    - set
    - name: value

# --- end of state config ---

test:
  cmd.run:
    - name: echo name1={{sls_params.name1}} name2={{sls_params.name2}} {{extra.name}}
    - cwd: /
''', sls='test')
        self.assertEqual(len(result), 3)
        self.assertTrue('test::sls_params' in result and 'test' in result)
        self.assertTrue('test::extra' in result)
        self.assertEqual(result['test']['cmd.run'][0]['name'],
                         'echo name1=value1 name2=value2 value')

    def test_sls_dir(self):
        result = self._render_sls('''
test:
  cmd.run:
    - name: echo sls_dir={{sls_dir}}
    - cwd: /
''', sls='path.to.sls')
        self.assertEqual(
            result['test']['cmd.run'][0]['name'],
            'echo sls_dir=path{0}to'.format(os.sep))

    def test_states_declared_with_shorthand_no_args(self):
        result = self._render_sls('''
test:
  cmd.run:
    - name: echo testing
    - cwd: /
test1:
  pkg.installed
test2:
  user.present
''')
        self.assertEqual(len(result), 3)
        for args in (result['test1']['pkg.installed'],
                     result['test2']['user.present']):
            self.assertTrue(isinstance(args, list))
            self.assertEqual(len(args), 0)
        self.assertEqual(result['test']['cmd.run'][0]['name'], 'echo testing')

    def test_adding_state_name_arg_for_dot_state_id(self):
        result = self._render_sls('''
.test:
  pkg.installed:
    - cwd: /
.test2:
  pkg.installed:
    - name: vim
''', sls='test')
        self.assertEqual(
            result['test::test']['pkg.installed'][0]['name'], 'test'
        )
        self.assertEqual(
            result['test::test2']['pkg.installed'][0]['name'], 'vim'
        )

    def test_state_prefix(self):
        result = self._render_sls('''
.test:
  cmd.run:
    - name: echo renamed
    - cwd: /

state_id:
  cmd:
    - run
    - name: echo not renamed
    - cwd: /
''', sls='test')
        self.assertEqual(len(result), 2)
        self.assertTrue('test::test' in result)
        self.assertTrue('state_id' in result)

    def test_dot_state_id_in_requisites(self):
        for req in REQUISITES:
            result = self._render_sls('''
.test:
  cmd.run:
    - name: echo renamed
    - cwd: /

state_id:
  cmd.run:
    - name: echo not renamed
    - cwd: /
    - {0}:
      - cmd: .test

    '''.format(req), sls='test')
            self.assertEqual(len(result), 2)
            self.assertTrue('test::test' in result)
            self.assertTrue('state_id' in result)
            self.assertEqual(
                result['state_id']['cmd.run'][2][req][0]['cmd'], 'test::test'
            )

    def test_relative_include_with_requisites(self):
        for req in REQUISITES:
            result = self._render_sls('''
include:
  - some.helper
  - .utils

state_id:
  cmd.run:
    - name: echo test
    - cwd: /
    - {0}:
      - cmd: .utils::some_state
'''.format(req), sls='test.work')
            self.assertEqual(result['include'][1], {'base': 'test.utils'})
            self.assertEqual(
                result['state_id']['cmd.run'][2][req][0]['cmd'],
                'test.utils::some_state'
            )

    def test_relative_include_and_extend(self):
        result = self._render_sls('''
include:
  - some.helper
  - .utils

extend:
  .utils::some_state:
    cmd.run:
      - name: echo overridden
    ''', sls='test.work')
        self.assertTrue('test.utils::some_state' in result['extend'])

    def test_multilevel_relative_include_with_requisites(self):
        for req in REQUISITES:
            result = self._render_sls('''
include:
  - .shared
  - ..utils
  - ...helper

state_id:
  cmd.run:
    - name: echo test
    - cwd: /
    - {0}:
      - cmd: ..utils::some_state
'''.format(req), sls='test.nested.work')
            self.assertEqual(result['include'][0],
                             {'base': 'test.nested.shared'})
            self.assertEqual(result['include'][1], {'base': 'test.utils'})
            self.assertEqual(result['include'][2], {'base': 'helper'})
            self.assertEqual(
                result['state_id']['cmd.run'][2][req][0]['cmd'],
                'test.utils::some_state'
            )

    def test_multilevel_relative_include_beyond_top_level(self):
        self.assertRaises(SaltRenderError, self._render_sls, '''
include:
  - ...shared
''', sls='test.work')

    def test_start_state_generation(self):
        result = self._render_sls('''
A:
  cmd.run:
    - name: echo hello
    - cwd: /
B:
  cmd.run:
    - name: echo world
    - cwd: /
''', sls='test', argline='-so yaml . jinja')
        self.assertEqual(len(result), 4)
        self.assertEqual(
            result['test::start']['stateconf.set'][0]['require_in'][0]['cmd'],
            'A'
        )

    def test_goal_state_generation(self):
        result = self._render_sls('''
{% for sid in "ABCDE": %}
{{sid}}:
  cmd.run:
    - name: echo this is {{sid}}
    - cwd: /
{% endfor %}

''', sls='test.goalstate', argline='yaml . jinja')
        self.assertEqual(len(result), len('ABCDE') + 1)

        reqs = result['test.goalstate::goal']['stateconf.set'][0]['require']
        self.assertEqual(
            set([next(six.itervalues(i)) for i in reqs]), set('ABCDE')
        )

    def test_implicit_require_with_goal_state(self):
        result = self._render_sls('''
{% for sid in "ABCDE": %}
{{sid}}:
  cmd.run:
    - name: echo this is {{sid}}
    - cwd: /
{% endfor %}

F:
  cmd.run:
    - name: echo this is F
    - cwd: /
    - require:
      - cmd: A
      - cmd: B

G:
  cmd.run:
    - name: echo this is G
    - cwd: /
    - require:
      - cmd: D
      - cmd: F
''', sls='test', argline='-o yaml . jinja')

        sids = 'ABCDEFG'[::-1]
        for i, sid in enumerate(sids):
            if i < len(sids) - 1:
                self.assertEqual(
                    result[sid]['cmd.run'][2]['require'][0]['cmd'],
                    sids[i + 1]
                )

        F_args = result['F']['cmd.run']
        self.assertEqual(len(F_args), 3)
        F_req = F_args[2]['require']
        self.assertEqual(len(F_req), 3)
        self.assertEqual(F_req[1]['cmd'], 'A')
        self.assertEqual(F_req[2]['cmd'], 'B')

        G_args = result['G']['cmd.run']
        self.assertEqual(len(G_args), 3)
        G_req = G_args[2]['require']
        self.assertEqual(len(G_req), 3)
        self.assertEqual(G_req[1]['cmd'], 'D')
        self.assertEqual(G_req[2]['cmd'], 'F')

        goal_args = result['test::goal']['stateconf.set']
        self.assertEqual(len(goal_args), 1)
        self.assertEqual(
            [next(six.itervalues(i)) for i in goal_args[0]['require']],
            list('ABCDEFG')
        )

    def test_slsdir(self):
        result = self._render_sls('''
formula/woot.sls:
  cmd.run:
    - name: echo {{ slspath }}
    - cwd: /
''', sls='formula.woot', argline='yaml . jinja')

        r = result['formula/woot.sls']['cmd.run'][0]['name']
        self.assertEqual(r, 'echo formula/woot')



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import
import collections

# Import Salt Testing Libs
from tests.support import mock
from tests.support import mixins
from tests.support.unit import skipIf, TestCase

# Import Salt Libs
import salt.roster.sshconfig as sshconfig

_SAMPLE_SSH_CONFIG = """
Host *
    User user.mcuserface

Host abc*
    IdentityFile ~/.ssh/id_rsa_abc

Host def*
    IdentityFile ~/.ssh/id_rsa_def

Host abc.asdfgfdhgjkl.com
    HostName 123.123.123.123

Host abc123.asdfgfdhgjkl.com
    HostName 123.123.123.124

Host def.asdfgfdhgjkl.com
    HostName 234.234.234.234
"""

_TARGET_ABC = collections.OrderedDict([
    ('user', 'user.mcuserface'),
    ('priv', '~/.ssh/id_rsa_abc'),
    ('host', 'abc.asdfgfdhgjkl.com')
])

_TARGET_ABC123 = collections.OrderedDict([
    ('user', 'user.mcuserface'),
    ('priv', '~/.ssh/id_rsa_abc'),
    ('host', 'abc123.asdfgfdhgjkl.com')
])

_TARGET_DEF = collections.OrderedDict([
    ('user', 'user.mcuserface'),
    ('priv', '~/.ssh/id_rsa_def'),
    ('host', 'def.asdfgfdhgjkl.com')
])

_ALL = {
    'abc.asdfgfdhgjkl.com': _TARGET_ABC,
    'abc123.asdfgfdhgjkl.com': _TARGET_ABC123,
    'def.asdfgfdhgjkl.com': _TARGET_DEF
}

_ABC_GLOB = {
    'abc.asdfgfdhgjkl.com': _TARGET_ABC,
    'abc123.asdfgfdhgjkl.com': _TARGET_ABC123
}


@skipIf(mock.NO_MOCK, mock.NO_MOCK_REASON)
class SSHConfigRosterTestCase(TestCase, mixins.LoaderModuleMockMixin):

    @classmethod
    def setUpClass(cls):
        cls.mock_fp = mock_fp = mock.mock_open(read_data=_SAMPLE_SSH_CONFIG)

    def setup_loader_modules(self):
        return {sshconfig: {}}

    def test_all(self):
        with mock.patch('salt.utils.files.fopen', self.mock_fp):
            with mock.patch('salt.roster.sshconfig._get_ssh_config_file'):
                targets = sshconfig.targets('*')
        self.assertEqual(targets, _ALL)

    def test_abc_glob(self):
        with mock.patch('salt.utils.files.fopen', self.mock_fp):
            with mock.patch('salt.roster.sshconfig._get_ssh_config_file'):
                targets = sshconfig.targets('abc*')
        self.assertEqual(targets, _ABC_GLOB)



# -*- coding: utf-8 -*-
'''
Test wrapper for running all KitchenSalt tests

All directories in 'tests/kitchen/' will be treated as a separate test under
the KitchenTestCase.
'''
from __future__ import absolute_import
import os

from tests.support.unit import TestCase, skipIf
import setup

import salt.utils.path
from salt.modules import cmdmod as cmd

CURRENT_DIR = os.path.dirname(os.path.realpath(__file__))


@skipIf(not salt.utils.path.which('bundle'), 'Bundler is not installed')
class KitchenTestCase(TestCase):
    '''
    Test kitchen environments
    '''
    @classmethod
    def setUpClass(cls):
        '''
        setup kitchen tests
        '''
        cls.topdir = '/' + os.path.join(*CURRENT_DIR.split('/')[:-2])
        cls.use_vt = int(os.environ.get('TESTS_LOG_LEVEL')) >= 5
        cmd.run('python setup.py sdist', cwd=cls.topdir)
        cmd.run('bundle install', cwd=CURRENT_DIR)
        cls.env = {
            'KITCHEN_YAML': os.path.join(CURRENT_DIR, '.kitchen.yml'),
            'SALT_SDIST_PATH': os.path.join(cls.topdir, 'dist', 'salt-{0}.tar.gz'.format(setup.__version__)),
        }

    @classmethod
    def tearDownClass(cls):
        del cls.topdir
        del cls.env

    def tearDown(self):
        cmd.run(
            'bundle exec kitchen destroy all',
            cwd=os.path.join(CURRENT_DIR, 'tests', self.testdir),
            env=self.env,
            use_vt=self.use_vt,
        )
        del self.testdir


def func_builder(testdir):
    def func(self):
        self.testdir = testdir
        if 'TESTS_XML_OUTPUT_DIR' in os.environ:
            self.env['TESTS_JUNIT_XML_PATH'] = '{0}/kitchen.tests.{1}.$KITCHEN_SUITE.$KITCHEN_PLATFORM.xml'.format(
                os.environ.get('TESTS_XML_OUTPUT_DIR'),
                self.testdir,
            )
        self.assertEqual(
            cmd.retcode(
                'bundle exec kitchen converge -c 999 all',
                cwd=os.path.join(CURRENT_DIR, 'tests', self.testdir),
                env=self.env,
                use_vt=self.use_vt,
            ),
            0
        )
        self.assertEqual(
            cmd.retcode(
                'bundle exec kitchen verify all',
                cwd=os.path.join(CURRENT_DIR, 'tests', self.testdir),
                env=self.env,
                use_vt=self.use_vt,
            ),
            0
        )
    return func


for testdir in os.listdir(os.path.join(CURRENT_DIR, 'tests')):
    setattr(KitchenTestCase, 'test_kitchen_{0}'.format(testdir), func_builder(testdir))



# -*- coding: utf-8 -*-
'''
    :codeauthor: Rupesh Tare <rupesht@saltstack.com>
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    Mock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)
try:
    import pytest
except ImportError as import_error:
    pytest = None

# Import Salt Libs
import salt.modules.localemod as localemod
from salt.exceptions import CommandExecutionError
from salt.ext import six


@skipIf(not pytest, False)
@skipIf(NO_MOCK, NO_MOCK_REASON)
class LocalemodTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.localemod
    '''
    locale_ctl_out = '''
   System Locale: LANG=de_DE.utf8
                  LANGUAGE=de_DE.utf8
       VC Keymap: n/a
      X11 Layout: us
       X11 Model: pc105
    '''
    locale_ctl_notset = '''
       System Locale: n/a

       VC Keymap: n/a
      X11 Layout: n/a
       X11 Model: n/a
    '''
    locale_ctl_out_empty = ''
    locale_ctl_out_broken = '''
    System error:Recursive traversal of loopback mount points
    '''
    locale_ctl_out_structure = '''
       Main: printers=We're upgrading /dev/null
             racks=hardware stress fractures
             failure=Ionisation from the air-conditioning
    Cow say: errors=We're out of slots on the server
             hardware=high pressure system failure
     Reason: The vendor put the bug there.
    '''

    def setup_loader_modules(self):
        return {localemod: {}}

    def test_list_avail(self):
        '''
        Test for Lists available (compiled) locales
        '''
        with patch.dict(localemod.__salt__,
                        {'cmd.run': MagicMock(return_value='A\nB')}):
            assert localemod.list_avail() == ['A', 'B']

    @patch('salt.utils.path.which', MagicMock(return_value="/usr/bin/localctl"))
    @patch('salt.modules.localemod.__salt__', {'cmd.run': MagicMock(return_value=locale_ctl_out)})
    def test_localectl_status_parser(self):
        '''
        Test localectl status parser.
        :return:
        '''
        out = localemod._localectl_status()
        assert isinstance(out, dict)
        for key in ['system_locale', 'vc_keymap', 'x11_layout', 'x11_model']:
            assert key in out
        assert isinstance(out['system_locale'], dict)
        assert 'LANG' in out['system_locale']
        assert 'LANGUAGE' in out['system_locale']
        assert out['system_locale']['LANG'] == out['system_locale']['LANGUAGE'] == 'de_DE.utf8'
        assert isinstance(out['vc_keymap'], dict)
        assert 'data' in out['vc_keymap']
        assert out['vc_keymap']['data'] is None
        assert isinstance(out['x11_layout'], dict)
        assert 'data' in out['x11_layout']
        assert out['x11_layout']['data'] == 'us'
        assert isinstance(out['x11_model'], dict)
        assert 'data' in out['x11_model']
        assert out['x11_model']['data'] == 'pc105'

    @patch('salt.utils.path.which', MagicMock(return_value="/usr/bin/localctl"))
    @patch('salt.modules.localemod.__salt__', {'cmd.run': MagicMock(return_value=locale_ctl_notset)})
    def test_localectl_status_parser_notset(self):
        '''
        Test localectl status parser.
        :return:
        '''
        out = localemod._localectl_status()
        assert isinstance(out, dict)
        for key in ['system_locale', 'vc_keymap', 'x11_layout']:
            assert key in out
        assert isinstance(out['system_locale'], dict)
        assert 'data' in out['system_locale']
        assert out['system_locale']['data'] is None
        assert isinstance(out['vc_keymap'], dict)
        assert 'data' in out['vc_keymap']
        assert out['vc_keymap']['data'] is None
        assert isinstance(out['x11_layout'], dict)
        assert 'data' in out['x11_layout']
        assert out['x11_layout']['data'] is None

    @patch('salt.modules.localemod.dbus', MagicMock())
    def test_dbus_locale_parser_matches(self):
        '''
        Test dbus locale status parser matching the results.
        :return:
        '''
        i_dbus = MagicMock()
        i_dbus.Get = MagicMock(return_value=['LANG=de_DE.utf8'])
        dbus = MagicMock(return_value=i_dbus)

        with patch('salt.modules.localemod.dbus.Interface', dbus):
            out = localemod._parse_dbus_locale()
            assert isinstance(out, dict)
            assert 'LANG' in out
            assert out['LANG'] == 'de_DE.utf8'

    @patch('salt.modules.localemod.dbus', MagicMock())
    @patch('salt.modules.localemod.log', MagicMock())
    def test_dbus_locale_parser_doesnot_matches(self):
        '''
        Test dbus locale status parser does not matching the results.
        :return:
        '''
        i_dbus = MagicMock()
        i_dbus.Get = MagicMock(return_value=['Fatal error right in front of screen'])
        dbus = MagicMock(return_value=i_dbus)

        with patch('salt.modules.localemod.dbus.Interface', dbus):
            out = localemod._parse_dbus_locale()
            assert isinstance(out, dict)
            assert 'LANG' not in out
            assert localemod.log.error.called
            msg = localemod.log.error.call_args[0][0] % localemod.log.error.call_args[0][1]
            assert msg == ('Odd locale parameter "Fatal error right in front of screen" detected in dbus locale output.'
                           ' This should not happen. You should probably investigate what caused this.')

    @patch('salt.utils.path.which', MagicMock(return_value=None))
    @patch('salt.modules.localemod.log', MagicMock())
    def test_localectl_status_parser_no_systemd(self):
        '''
        Test localectl status parser raises an exception if no systemd installed.
        :return:
        '''
        with pytest.raises(CommandExecutionError) as err:
            localemod._localectl_status()
        assert 'Unable to find "localectl"' in six.text_type(err)
        assert not localemod.log.debug.called

    @patch('salt.utils.path.which', MagicMock(return_value="/usr/bin/localctl"))
    @patch('salt.modules.localemod.__salt__', {'cmd.run': MagicMock(return_value=locale_ctl_out_empty)})
    def test_localectl_status_parser_empty(self):
        with pytest.raises(CommandExecutionError) as err:
            localemod._localectl_status()
        assert 'Unable to parse result of "localectl"' in six.text_type(err)

    @patch('salt.utils.path.which', MagicMock(return_value="/usr/bin/localctl"))
    @patch('salt.modules.localemod.__salt__', {'cmd.run': MagicMock(return_value=locale_ctl_out_broken)})
    def test_localectl_status_parser_broken(self):
        with pytest.raises(CommandExecutionError) as err:
            localemod._localectl_status()
        assert 'Unable to parse result of "localectl"' in six.text_type(err)

    @patch('salt.utils.path.which', MagicMock(return_value="/usr/bin/localctl"))
    @patch('salt.modules.localemod.__salt__', {'cmd.run': MagicMock(return_value=locale_ctl_out_structure)})
    def test_localectl_status_parser_structure(self):
        out = localemod._localectl_status()
        assert isinstance(out, dict)
        for key in ['main', 'cow_say']:
            assert isinstance(out[key], dict)
            for in_key in out[key]:
                assert isinstance(out[key][in_key], six.text_type)
        assert isinstance(out['reason']['data'], six.text_type)

    @patch('salt.utils.path.which', MagicMock(return_value="/usr/bin/localctl"))
    @patch('salt.modules.localemod.__grains__', {'os_family': 'Ubuntu', 'osmajorrelease': 42})
    @patch('salt.modules.localemod.dbus', None)
    @patch('salt.modules.localemod._parse_dbus_locale', MagicMock(return_value={'LANG': 'en_US.utf8'}))
    @patch('salt.modules.localemod._localectl_status', MagicMock(return_value={'system_locale': {'LANG': 'de_DE.utf8'}}))
    @patch('salt.utils.systemd.booted', MagicMock(return_value=True))
    def test_get_locale_with_systemd_nodbus(self):
        '''
        Test getting current system locale with systemd but no dbus available.
        :return:
        '''
        assert localemod.get_locale() == 'de_DE.utf8'

    @patch('salt.utils.path.which', MagicMock(return_value="/usr/bin/localctl"))
    @patch('salt.modules.localemod.__grains__', {'os_family': 'Ubuntu', 'osmajorrelease': 42})
    @patch('salt.modules.localemod.dbus', True)
    @patch('salt.modules.localemod._parse_dbus_locale', MagicMock(return_value={'LANG': 'en_US.utf8'}))
    @patch('salt.modules.localemod._localectl_status', MagicMock(return_value={'system_locale': {'LANG': 'de_DE.utf8'}}))
    @patch('salt.utils.systemd.booted', MagicMock(return_value=True))
    def test_get_locale_with_systemd_and_dbus(self):
        '''
        Test getting current system locale with systemd and dbus available.
        :return:
        '''
        assert localemod.get_locale() == 'en_US.utf8'

    @patch('salt.utils.path.which', MagicMock(return_value="/usr/bin/localctl"))
    @patch('salt.modules.localemod.__grains__', {'os_family': 'Suse', 'osmajorrelease': 12})
    @patch('salt.modules.localemod.dbus', True)
    @patch('salt.modules.localemod._parse_dbus_locale', MagicMock(return_value={'LANG': 'en_US.utf8'}))
    @patch('salt.modules.localemod._localectl_status', MagicMock(return_value={'system_locale': {'LANG': 'de_DE.utf8'}}))
    @patch('salt.modules.localemod.__salt__', {'cmd.run': MagicMock()})
    @patch('salt.utils.systemd.booted', MagicMock(return_value=True))
    def test_get_locale_with_systemd_and_dbus_sle12(self):
        '''
        Test getting current system locale with systemd and dbus available on SLE12.
        :return:
        '''
        localemod.get_locale()
        assert localemod.__salt__['cmd.run'].call_args[0][0] == 'grep "^RC_LANG" /etc/sysconfig/language'

    @patch('salt.utils.path.which', MagicMock(return_value=None))
    @patch('salt.modules.localemod.__grains__', {'os_family': 'RedHat', 'osmajorrelease': 12})
    @patch('salt.modules.localemod.dbus', None)
    @patch('salt.modules.localemod.__salt__', {'cmd.run': MagicMock()})
    @patch('salt.utils.systemd.booted', MagicMock(return_value=False))
    def test_get_locale_with_no_systemd_redhat(self):
        '''
        Test getting current system locale with systemd and dbus available on RedHat.
        :return:
        '''
        localemod.get_locale()
        assert localemod.__salt__['cmd.run'].call_args[0][0] == 'grep "^LANG=" /etc/sysconfig/i18n'

    @patch('salt.utils.path.which', MagicMock(return_value=None))
    @patch('salt.modules.localemod.__grains__', {'os_family': 'Debian', 'osmajorrelease': 12})
    @patch('salt.modules.localemod.dbus', None)
    @patch('salt.modules.localemod.__salt__', {'cmd.run': MagicMock()})
    @patch('salt.utils.systemd.booted', MagicMock(return_value=False))
    def test_get_locale_with_no_systemd_debian(self):
        '''
        Test getting current system locale with systemd and dbus available on Debian.
        :return:
        '''
        localemod.get_locale()
        assert localemod.__salt__['cmd.run'].call_args[0][0] == 'grep "^LANG=" /etc/default/locale'

    @patch('salt.utils.path.which', MagicMock(return_value=None))
    @patch('salt.modules.localemod.__grains__', {'os_family': 'Gentoo', 'osmajorrelease': 12})
    @patch('salt.modules.localemod.dbus', None)
    @patch('salt.modules.localemod.__salt__', {'cmd.run': MagicMock()})
    @patch('salt.utils.systemd.booted', MagicMock(return_value=False))
    def test_get_locale_with_no_systemd_gentoo(self):
        '''
        Test getting current system locale with systemd and dbus available on Gentoo.
        :return:
        '''
        localemod.get_locale()
        assert localemod.__salt__['cmd.run'].call_args[0][0] == 'eselect --brief locale show'

    @patch('salt.utils.path.which', MagicMock(return_value=None))
    @patch('salt.modules.localemod.__grains__', {'os_family': 'Solaris', 'osmajorrelease': 12})
    @patch('salt.modules.localemod.dbus', None)
    @patch('salt.modules.localemod.__salt__', {'cmd.run': MagicMock()})
    @patch('salt.utils.systemd.booted', MagicMock(return_value=False))
    def test_get_locale_with_no_systemd_solaris(self):
        '''
        Test getting current system locale with systemd and dbus available on Solaris.
        :return:
        '''
        localemod.get_locale()
        assert localemod.__salt__['cmd.run'].call_args[0][0] == 'grep "^LANG=" /etc/default/init'

    @patch('salt.utils.path.which', MagicMock(return_value=None))
    @patch('salt.modules.localemod.__grains__', {'os_family': 'BSD', 'osmajorrelease': 8, 'oscodename': 'DrunkDragon'})
    @patch('salt.modules.localemod.dbus', None)
    @patch('salt.modules.localemod.__salt__', {'cmd.run': MagicMock()})
    @patch('salt.utils.systemd.booted', MagicMock(return_value=False))
    def test_get_locale_with_no_systemd_unknown(self):
        '''
        Test getting current system locale with systemd and dbus available on Gentoo.
        :return:
        '''
        with pytest.raises(CommandExecutionError) as err:
            localemod.get_locale()
        assert '"DrunkDragon" is unsupported' in six.text_type(err)

    @patch('salt.utils.path.which', MagicMock(return_value="/usr/bin/localctl"))
    @patch('salt.modules.localemod.__grains__', {'os_family': 'Ubuntu', 'osmajorrelease': 42})
    @patch('salt.modules.localemod.dbus', None)
    @patch('salt.utils.systemd.booted', MagicMock(return_value=True))
    @patch('salt.modules.localemod._localectl_set', MagicMock())
    def test_set_locale_with_systemd_nodbus(self):
        '''
        Test setting current system locale with systemd but no dbus available.
        :return:
        '''
        loc = 'de_DE.utf8'
        localemod.set_locale(loc)
        assert localemod._localectl_set.call_args[0][0] == 'de_DE.utf8'

    @patch('salt.utils.path.which', MagicMock(return_value="/usr/bin/localctl"))
    @patch('salt.modules.localemod.__grains__', {'os_family': 'Ubuntu', 'osmajorrelease': 42})
    @patch('salt.modules.localemod.dbus', True)
    @patch('salt.utils.systemd.booted', MagicMock(return_value=True))
    @patch('salt.modules.localemod._localectl_set', MagicMock())
    def test_set_locale_with_systemd_and_dbus(self):
        '''
        Test setting current system locale with systemd and dbus available.
        :return:
        '''
        loc = 'de_DE.utf8'
        localemod.set_locale(loc)
        assert localemod._localectl_set.call_args[0][0] == 'de_DE.utf8'

    @patch('salt.utils.path.which', MagicMock(return_value="/usr/bin/localctl"))
    @patch('salt.modules.localemod.__grains__', {'os_family': 'Suse', 'osmajorrelease': 12})
    @patch('salt.modules.localemod.dbus', True)
    @patch('salt.modules.localemod.__salt__', MagicMock())
    @patch('salt.modules.localemod._localectl_set', MagicMock())
    @patch('salt.utils.systemd.booted', MagicMock(return_value=True))
    def test_set_locale_with_systemd_and_dbus_sle12(self):
        '''
        Test setting current system locale with systemd and dbus available on SLE12.
        :return:
        '''
        loc = 'de_DE.utf8'
        localemod.set_locale(loc)
        assert not localemod._localectl_set.called
        assert localemod.__salt__['file.replace'].called
        assert localemod.__salt__['file.replace'].call_args[0][0] == '/etc/sysconfig/language'
        assert localemod.__salt__['file.replace'].call_args[0][1] == '^RC_LANG=.*'
        assert localemod.__salt__['file.replace'].call_args[0][2] == 'RC_LANG="{}"'.format(loc)

    @patch('salt.utils.path.which', MagicMock(return_value=None))
    @patch('salt.modules.localemod.__grains__', {'os_family': 'RedHat', 'osmajorrelease': 42})
    @patch('salt.modules.localemod.dbus', None)
    @patch('salt.modules.localemod.__salt__', MagicMock())
    @patch('salt.modules.localemod._localectl_set', MagicMock())
    @patch('salt.utils.systemd.booted', MagicMock(return_value=False))
    def test_set_locale_with_no_systemd_redhat(self):
        '''
        Test setting current system locale with systemd and dbus available on RedHat.
        :return:
        '''
        loc = 'de_DE.utf8'
        localemod.set_locale(loc)
        assert not localemod._localectl_set.called
        assert localemod.__salt__['file.replace'].called
        assert localemod.__salt__['file.replace'].call_args[0][0] == '/etc/sysconfig/i18n'
        assert localemod.__salt__['file.replace'].call_args[0][1] == '^LANG=.*'
        assert localemod.__salt__['file.replace'].call_args[0][2] == 'LANG="{}"'.format(loc)

    @patch('salt.utils.path.which', MagicMock(return_value='/usr/sbin/update-locale'))
    @patch('salt.modules.localemod.__grains__', {'os_family': 'Debian', 'osmajorrelease': 42})
    @patch('salt.modules.localemod.dbus', None)
    @patch('salt.modules.localemod.__salt__', MagicMock())
    @patch('salt.modules.localemod._localectl_set', MagicMock())
    @patch('salt.utils.systemd.booted', MagicMock(return_value=False))
    def test_set_locale_with_no_systemd_debian(self):
        '''
        Test setting current system locale with systemd and dbus available on Debian.
        :return:
        '''
        loc = 'de_DE.utf8'
        localemod.set_locale(loc)
        assert not localemod._localectl_set.called
        assert localemod.__salt__['file.replace'].called
        assert localemod.__salt__['file.replace'].call_args[0][0] == '/etc/default/locale'
        assert localemod.__salt__['file.replace'].call_args[0][1] == '^LANG=.*'
        assert localemod.__salt__['file.replace'].call_args[0][2] == 'LANG="{}"'.format(loc)

    @patch('salt.utils.path.which', MagicMock(return_value=None))
    @patch('salt.utils.path.which', MagicMock(return_value=None))
    @patch('salt.modules.localemod.__grains__', {'os_family': 'Debian', 'osmajorrelease': 42})
    @patch('salt.modules.localemod.dbus', None)
    @patch('salt.modules.localemod.__salt__', MagicMock())
    @patch('salt.modules.localemod._localectl_set', MagicMock())
    @patch('salt.utils.systemd.booted', MagicMock(return_value=False))
    def test_set_locale_with_no_systemd_debian_no_update_locale(self):
        '''
        Test setting current system locale with systemd and dbus available on Debian but update-locale is not installed.
        :return:
        '''
        loc = 'de_DE.utf8'
        with pytest.raises(CommandExecutionError) as err:
            localemod.set_locale(loc)
        assert not localemod._localectl_set.called
        assert 'Cannot set locale: "update-locale" was not found.' in six.text_type(err)

    @patch('salt.utils.path.which', MagicMock(return_value=None))
    @patch('salt.modules.localemod.__grains__', {'os_family': 'Gentoo', 'osmajorrelease': 42})
    @patch('salt.modules.localemod.dbus', None)
    @patch('salt.modules.localemod.__salt__', MagicMock())
    @patch('salt.modules.localemod._localectl_set', MagicMock())
    @patch('salt.utils.systemd.booted', MagicMock(return_value=False))
    def test_set_locale_with_no_systemd_gentoo(self):
        '''
        Test setting current system locale with systemd and dbus available on Gentoo.
        :return:
        '''
        loc = 'de_DE.utf8'
        localemod.set_locale(loc)
        assert not localemod._localectl_set.called
        assert localemod.__salt__['cmd.retcode'].call_args[0][0] == 'eselect --brief locale set de_DE.utf8'

    @patch('salt.utils.path.which', MagicMock(return_value=None))
    @patch('salt.modules.localemod.__grains__', {'os_family': 'Solaris', 'osmajorrelease': 42})
    @patch('salt.modules.localemod.dbus', None)
    @patch('salt.modules.localemod.__salt__', {'locale.list_avail': MagicMock(return_value=['de_DE.utf8']),
                                               'file.replace': MagicMock()})
    @patch('salt.modules.localemod._localectl_set', MagicMock())
    @patch('salt.utils.systemd.booted', MagicMock(return_value=False))
    def test_set_locale_with_no_systemd_solaris_with_list_avail(self):
        '''
        Test setting current system locale with systemd and dbus available on Solaris.
        The list_avail returns the proper locale.
        :return:
        '''
        loc = 'de_DE.utf8'
        localemod.set_locale(loc)
        assert not localemod._localectl_set.called
        assert localemod.__salt__['file.replace'].called
        assert localemod.__salt__['file.replace'].call_args[0][0] == '/etc/default/init'
        assert localemod.__salt__['file.replace'].call_args[0][1] == '^LANG=.*'
        assert localemod.__salt__['file.replace'].call_args[0][2] == 'LANG="{}"'.format(loc)

    @patch('salt.utils.path.which', MagicMock(return_value=None))
    @patch('salt.modules.localemod.__grains__', {'os_family': 'Solaris', 'osmajorrelease': 42})
    @patch('salt.modules.localemod.dbus', None)
    @patch('salt.modules.localemod.__salt__', {'locale.list_avail': MagicMock(return_value=['en_GB.utf8']),
                                               'file.replace': MagicMock()})
    @patch('salt.modules.localemod._localectl_set', MagicMock())
    @patch('salt.utils.systemd.booted', MagicMock(return_value=False))
    def test_set_locale_with_no_systemd_solaris_without_list_avail(self):
        '''
        Test setting current system locale with systemd and dbus is not available on Solaris.
        The list_avail does not return the proper locale.
        :return:
        '''
        loc = 'de_DE.utf8'
        assert not localemod.set_locale(loc)
        assert not localemod._localectl_set.called
        assert not localemod.__salt__['file.replace'].called

    @patch('salt.utils.path.which', MagicMock(return_value=None))
    @patch('salt.modules.localemod.__grains__', {'os_family': 'BSD', 'osmajorrelease': 42})
    @patch('salt.modules.localemod.dbus', None)
    @patch('salt.modules.localemod.__salt__', {'locale.list_avail': MagicMock(return_value=['en_GB.utf8']),
                                               'file.replace': MagicMock()})
    @patch('salt.modules.localemod._localectl_set', MagicMock())
    @patch('salt.utils.systemd.booted', MagicMock(return_value=False))
    def test_set_locale_with_no_systemd_unknown(self):
        '''
        Test setting current system locale without systemd on unknown system.
        :return:
        '''
        with pytest.raises(CommandExecutionError) as err:
            localemod.set_locale('de_DE.utf8')
        assert 'Unsupported platform' in six.text_type(err)

    @patch('salt.utils.locales.normalize_locale', MagicMock(return_value='en_US.UTF-8 UTF-8'))
    @patch('salt.modules.localemod.__salt__', {'locale.list_avail': MagicMock(return_value=['A', 'B'])})
    def test_avail(self):
        '''
        Test for Check if a locale is available
        '''
        assert localemod.avail('locale')

    @patch('salt.modules.localemod.log', MagicMock())
    @patch('salt.utils.path.which', MagicMock(return_value='/some/dir/path'))
    @patch('salt.modules.localemod.__grains__', {'os': 'Debian'})
    @patch('salt.modules.localemod.__salt__', {'file.search': MagicMock(return_value=False)})
    def test_gen_locale_not_valid(self):
        '''
        Tests the return of gen_locale when the provided locale is not found
        '''
        assert not localemod.gen_locale('foo')
        assert localemod.log.error.called
        msg = localemod.log.error.call_args[0][0] % (localemod.log.error.call_args[0][1],
                                                     localemod.log.error.call_args[0][2])
        assert msg == 'The provided locale "foo" is not found in /usr/share/i18n/SUPPORTED'

    @patch('salt.modules.localemod.log', MagicMock())
    @patch('salt.modules.localemod.__grains__', {'os_family': 'Suse'})
    @patch('os.listdir', MagicMock(return_value=[]))
    @patch('salt.utils.locales.join_locale', MagicMock(return_value='en_GB.utf8'))
    def test_gen_locale_suse_invalid(self):
        '''
        Tests the location where gen_locale is searching for generated paths.
        :return:
        '''
        assert not localemod.gen_locale('de_DE.utf8')
        assert localemod.log.error.called
        msg = localemod.log.error.call_args[0][0] % (localemod.log.error.call_args[0][1],
                                                     localemod.log.error.call_args[0][2])
        assert localemod.os.listdir.call_args[0][0] == '/usr/share/locale'
        assert msg == 'The provided locale "en_GB.utf8" is not found in /usr/share/locale'

    @patch('salt.modules.localemod.log', MagicMock())
    @patch('salt.modules.localemod.__grains__', {'os_family': 'Suse'})
    @patch('salt.modules.localemod.__salt__', {'cmd.run_all': MagicMock(return_value={'retcode': 0})})
    @patch('os.listdir', MagicMock(return_value=['de_DE']))
    @patch('os.path.exists', MagicMock(return_value=False))
    @patch('salt.utils.locales.join_locale', MagicMock(return_value='de_DE.utf8'))
    @patch('salt.utils.path.which', MagicMock(side_effect=[None, '/usr/bin/localedef']))
    def test_gen_locale_suse_valid(self):
        '''
        Tests the location where gen_locale is calling localedef on Suse os-family.
        :return:
        '''
        localemod.gen_locale('de_DE.utf8')
        assert localemod.__salt__['cmd.run_all'].call_args[0][0] == ['localedef', '--force', '-i', 'de_DE',
                                                                     '-f', 'utf8', 'de_DE.utf8', '--quiet']

    @patch('salt.modules.localemod.log', MagicMock())
    @patch('salt.modules.localemod.__grains__', {'os_family': 'Suse'})
    @patch('salt.modules.localemod.__salt__', {'cmd.run_all': MagicMock(return_value={'retcode': 0})})
    @patch('os.listdir', MagicMock(return_value=['de_DE']))
    @patch('os.path.exists', MagicMock(return_value=False))
    @patch('salt.utils.locales.join_locale', MagicMock(return_value='de_DE.utf8'))
    @patch('salt.utils.path.which', MagicMock(return_value=None))
    def test_gen_locale_suse_localedef_error_handling(self):
        '''
        Tests the location where gen_locale is handling error while calling not installed localedef on Suse os-family.
        :return:
        '''
        with pytest.raises(CommandExecutionError) as err:
            localemod.gen_locale('de_DE.utf8')
        assert 'Command "locale-gen" or "localedef" was not found on this system.' in six.text_type(err)

    def test_gen_locale_debian(self):
        '''
        Tests the return of successful gen_locale on Debian system
        '''
        ret = {'stdout': 'saltines', 'stderr': 'biscuits', 'retcode': 0, 'pid': 1337}
        with patch.dict(localemod.__grains__, {'os': 'Debian'}), \
                patch('salt.utils.path.which', MagicMock(return_value='/some/dir/path')), \
                patch.dict(localemod.__salt__,
                           {'file.search': MagicMock(return_value=True),
                            'file.replace': MagicMock(return_value=True),
                            'cmd.run_all': MagicMock(return_value=ret)}):
            assert localemod.gen_locale('en_US.UTF-8 UTF-8')

    def test_gen_locale_debian_no_charmap(self):
        '''
        Tests the return of successful gen_locale on Debian system without a charmap
        '''
        ret = {'stdout': 'saltines', 'stderr': 'biscuits', 'retcode': 0, 'pid': 1337}
        with patch.dict(localemod.__grains__, {'os': 'Debian'}), \
                patch('salt.utils.path.which', MagicMock(return_value='/some/dir/path')), \
                patch.dict(localemod.__salt__,
                           {'file.search': lambda s, p, flags: not len(p.split()) == 1,
                            'file.replace': MagicMock(return_value=True),
                            'cmd.run_all': MagicMock(return_value=ret)}):
            assert localemod.gen_locale('en_US.UTF-8')

    def test_gen_locale_ubuntu(self):
        '''
        Test the return of successful gen_locale on Ubuntu system
        '''
        ret = {'stdout': 'saltines', 'stderr': 'biscuits', 'retcode': 0, 'pid': 1337}
        with patch.dict(localemod.__salt__,
                        {'file.replace': MagicMock(return_value=True),
                         'file.touch': MagicMock(return_value=None),
                         'file.append': MagicMock(return_value=None),
                         'cmd.run_all': MagicMock(return_value=ret)}), \
                patch('salt.utils.path.which', MagicMock(return_value='/some/dir/path')), \
                patch('os.listdir', MagicMock(return_value=['en_US'])), \
                patch.dict(localemod.__grains__, {'os': 'Ubuntu'}):
            assert localemod.gen_locale('en_US.UTF-8')

    def test_gen_locale_gentoo(self):
        '''
        Tests the return of successful gen_locale on Gentoo system
        '''
        ret = {'stdout': 'saltines', 'stderr': 'biscuits', 'retcode': 0, 'pid': 1337}
        with patch.dict(localemod.__grains__, {'os_family': 'Gentoo'}), \
                patch('salt.utils.path.which', MagicMock(return_value='/some/dir/path')), \
                patch('os.listdir', MagicMock(return_value=['en_US.UTF-8'])), \
                patch.dict(localemod.__salt__,
                           {'file.search': MagicMock(return_value=True),
                            'file.replace': MagicMock(return_value=True),
                            'cmd.run_all': MagicMock(return_value=ret)}):
            assert localemod.gen_locale('en_US.UTF-8 UTF-8')

    def test_gen_locale_gentoo_no_charmap(self):
        '''
        Tests the return of successful gen_locale on Gentoo system without a charmap
        '''
        def file_search(search, pattern, flags):
            '''
            mock file.search
            '''
            if len(pattern.split()) == 1:
                return False
            else:  # charmap was supplied
                return True

        ret = {'stdout': 'saltines', 'stderr': 'biscuits', 'retcode': 0, 'pid': 1337}
        with patch.dict(localemod.__grains__, {'os_family': 'Gentoo'}), \
                patch('salt.utils.path.which', MagicMock(return_value='/some/dir/path')), \
                patch('os.listdir', MagicMock(return_value=['en_US.UTF-8'])), \
                patch.dict(localemod.__salt__,
                           {'file.search': file_search,
                            'file.replace': MagicMock(return_value=True),
                            'cmd.run_all': MagicMock(return_value=ret)}):
            assert localemod.gen_locale('en_US.UTF-8')

    def test_gen_locale(self):
        '''
        Tests the return of successful gen_locale
        '''
        ret = {'stdout': 'saltines', 'stderr': 'biscuits', 'retcode': 0, 'pid': 1337}
        with patch.dict(localemod.__salt__,
                        {'cmd.run_all': MagicMock(return_value=ret),
                         'file.replace': MagicMock()}), \
                patch('salt.utils.path.which', MagicMock(return_value='/some/dir/path')), \
                patch('os.listdir', MagicMock(return_value=['en_US'])):
            assert localemod.gen_locale('en_US.UTF-8')

    def test_gen_locale_verbose(self):
        '''
        Tests the return of successful gen_locale
        '''
        ret = {'stdout': 'saltines', 'stderr': 'biscuits', 'retcode': 0, 'pid': 1337}
        with patch.dict(localemod.__salt__,
                        {'cmd.run_all': MagicMock(return_value=ret),
                         'file.replace': MagicMock()}), \
                patch('salt.utils.path.which', MagicMock(return_value='/some/dir/path')), \
                patch('os.listdir', MagicMock(return_value=['en_US'])):
            assert localemod.gen_locale('en_US.UTF-8', verbose=True) == ret

    @patch('salt.utils.path.which', MagicMock(return_value="/usr/bin/localctl"))
    def test_parse_localectl(self):
        localectl_out = ('   System Locale: LANG=en_US.UTF-8\n'
                         '                  LANGUAGE=en_US:en\n'
                         '       VC Keymap: n/a')
        mock_cmd = Mock(return_value=localectl_out)
        with patch.dict(localemod.__salt__, {'cmd.run': mock_cmd}):
            ret = localemod._localectl_status()['system_locale']
            assert {'LANG': 'en_US.UTF-8', 'LANGUAGE': 'en_US:en'} == ret
